import re
from collections import OrderedDict
from functools import reduce

from convertdate import persian

from dateparser.calendars import non_gregorian_parser


class PersianDate:
    def __init__(self, year, month, day):
        self.year = year
        self.month = month
        self.day = day

    def weekday(self):
        for week in persian.monthcalendar(self.year, self.month):
            for idx, day in enumerate(week):
                if day == self.day:
                    return idx


class jalali_parser(non_gregorian_parser):
    calendar_converter = persian
    default_year = 1348
    default_month = 1
    default_day = 1
    non_gregorian_date_cls = PersianDate

    _digits = {
        "۰": 0,
        "۱": 1,
        "۲": 2,
        "۳": 3,
        "۴": 4,
        "۵": 5,
        "۶": 6,
        "۷": 7,
        "۸": 8,
        "۹": 9,
    }

    _months = OrderedDict(
        [
            # pinglish : (persian literals, month index, number of days)
            ("Farvardin", (1, 31, ["فروردین"])),
            ("Ordibehesht", (2, 31, ["اردیبهشت"])),
            ("Khordad", (3, 31, ["خرداد"])),
            ("Tir", (4, 31, ["تیر"])),
            ("Mordad", (5, 31, ["امرداد", "مرداد"])),
            ("Shahrivar", (6, 31, ["شهریور", "شهريور"])),
            ("Mehr", (7, 30, ["مهر"])),
            ("Aban", (8, 30, ["آبان"])),
            ("Azar", (9, 30, ["آذر"])),
            ("Dey", (10, 30, ["دی"])),
            ("Bahman", (11, 30, ["بهمن", "بهن"])),
            ("Esfand", (12, 29, ["اسفند"])),
        ]
    )

    _weekdays = OrderedDict(
        [
            ("Sunday", ["یکشنبه"]),
            ("Monday", ["دوشنبه"]),
            ("Tuesday", ["سهشنبه", "سه شنبه"]),
            ("Wednesday", ["چهارشنبه", "چهار شنبه"]),
            ("Thursday", ["پنجشنبه", "پنج شنبه"]),
            ("Friday", ["جمعه"]),
            ("Saturday", ["روز شنبه", "شنبه"]),
        ]
    )

    _number_letters = {
        0: ["صفر"],
        1: ["یک", "اول"],
        2: ["دو"],
        3: ["سه", "سو"],
        4: ["چهار"],
        5: ["پنج"],
        6: ["شش"],
        7: ["هفت"],
        8: ["هشت"],
        9: ["نه"],
        10: ["ده"],
        11: ["یازده"],
        12: ["دوازده"],
        13: ["سیزده"],
        14: ["چهارده"],
        15: ["پانزده"],
        16: ["شانزده"],
        17: ["هفده"],
        18: ["هجده"],
        19: ["نوزده"],
        20: ["بیست"],
        21: ["بیست و یک"],
        22: ["بیست و دو", "بیست ثانیه"],
        23: ["بیست و سه", "بیست و سو"],
        24: ["بیست و چهار"],
        25: ["بیست و پنج"],
        26: ["بیست و شش"],
        27: ["بیست و هفت"],
        28: ["بیست و هشت"],
        29: ["بیست و نه"],
        30: ["سی"],
        31: ["سی و یک"],
    }

    @classmethod
    def _replace_digits(cls, source):
        result = source
        for pers_digit, number in cls._digits.items():
            result = result.replace(pers_digit, str(number))
        return result

    @classmethod
    def _replace_months(cls, source):
        result = source
        for pers, latin in reduce(
            lambda a, b: a + b,
            [
                [(value, month) for value in repl[-1]]
                for month, repl in cls._months.items()
            ],
        ):
            result = result.replace(pers, latin)
        return result

    @classmethod
    def _replace_weekdays(cls, source):
        result = source
        for pers, latin in reduce(
            lambda a, b: a + b,
            [
                [(value, weekday) for value in repl]
                for weekday, repl in cls._weekdays.items()
            ],
        ):
            result = result.replace(pers, latin)
        return result

    @classmethod
    def _replace_time(cls, source):
        def only_numbers(match_obj):
            matched_string = match_obj.group()
            return re.sub(r"\D", " ", matched_string)

        hour_pattern = r"ساعت\s+\d{2}"
        minute_pattern = r"\d{2}\s+دقیقه"
        second_pattern = r"\d{2}\s+ثانیه"
        result = re.sub(hour_pattern, only_numbers, source)
        result = re.sub(minute_pattern, only_numbers, result)
        result = re.sub(second_pattern, only_numbers, result)
        result = re.sub(r"\s+و\s+", ":", result)
        result = result.replace("ساعت", "")
        return result

    @classmethod
    def _replace_days(cls, source):
        result = re.sub(
            r"ام|م|ین", "", source
        )  # removes persian variant of th/first/second/third
        day_pairs = list(cls._number_letters.items())

        def comp_key(tup):
            return tup[0]

        day_pairs.sort(key=comp_key, reverse=True)

        thirteen, thirty = day_pairs[-14], day_pairs[1]
        day_pairs[-14] = thirty
        day_pairs[1] = thirteen

        for persian_number, number in reduce(
            lambda a, b: a + b,
            [[(val, repl) for val in persian_word] for repl, persian_word in day_pairs],
        ):
            result = result.replace(persian_number, str(number))
        return result

    def handle_two_digit_year(self, year):
        if year > 60:
            return year + 1300
        else:
            return year + 1400



info = {
    "name": "kk",
    "date_order": "DMY",
    "january": [
        "қаң",
        "қаңтар"
    ],
    "february": [
        "ақп",
        "ақпан"
    ],
    "march": [
        "нау",
        "наурыз"
    ],
    "april": [
        "сәу",
        "сәуір"
    ],
    "may": [
        "мам",
        "мамыр"
    ],
    "june": [
        "мау",
        "маусым"
    ],
    "july": [
        "шіл",
        "шілде"
    ],
    "august": [
        "там",
        "тамыз"
    ],
    "september": [
        "қыр",
        "қыркүйек"
    ],
    "october": [
        "қаз",
        "қазан"
    ],
    "november": [
        "қар",
        "қараша"
    ],
    "december": [
        "жел",
        "желтоқсан"
    ],
    "monday": [
        "дс",
        "дүйсенбі"
    ],
    "tuesday": [
        "сейсенбі",
        "сс"
    ],
    "wednesday": [
        "ср",
        "сәрсенбі"
    ],
    "thursday": [
        "бейсенбі",
        "бс"
    ],
    "friday": [
        "жм",
        "жұма"
    ],
    "saturday": [
        "сб",
        "сенбі"
    ],
    "sunday": [
        "жексенбі",
        "жс"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "ж",
        "жыл"
    ],
    "month": [
        "ай"
    ],
    "week": [
        "ап",
        "апта"
    ],
    "day": [
        "күн"
    ],
    "hour": [
        "сағ",
        "сағат"
    ],
    "minute": [
        "мин",
        "минут"
    ],
    "second": [
        "с",
        "секунд"
    ],
    "relative-type": {
        "0 day ago": [
            "бүгін"
        ],
        "0 hour ago": [
            "осы сағат"
        ],
        "0 minute ago": [
            "осы минут"
        ],
        "0 month ago": [
            "осы ай"
        ],
        "0 second ago": [
            "қазір"
        ],
        "0 week ago": [
            "осы апта"
        ],
        "0 year ago": [
            "биылғы жыл"
        ],
        "1 day ago": [
            "кеше"
        ],
        "1 month ago": [
            "өткен ай"
        ],
        "1 week ago": [
            "өткен апта"
        ],
        "1 year ago": [
            "былтырғы жыл"
        ],
        "in 1 day": [
            "ертең"
        ],
        "in 1 month": [
            "келесі ай"
        ],
        "in 1 week": [
            "келесі апта"
        ],
        "in 1 year": [
            "келесі жыл"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) күн бұрын"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) сағ бұрын",
            "(\\d+[.,]?\\d*) сағат бұрын"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) мин бұрын",
            "(\\d+[.,]?\\d*) минут бұрын"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) ай бұрын"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) сек бұрын",
            "(\\d+[.,]?\\d*) секунд бұрын"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) ап бұрын",
            "(\\d+[.,]?\\d*) апта бұрын"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) ж бұрын",
            "(\\d+[.,]?\\d*) жыл бұрын"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) күннен кейін"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) сағ кейін",
            "(\\d+[.,]?\\d*) сағаттан кейін"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) мин кейін",
            "(\\d+[.,]?\\d*) минуттан кейін"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) айдан кейін"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) сек кейін",
            "(\\d+[.,]?\\d*) секундтан кейін"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) ап кейін",
            "(\\d+[.,]?\\d*) аптадан кейін"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) ж кейін",
            "(\\d+[.,]?\\d*) жылдан кейін"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import calendar
import logging
import types
import unicodedata
from collections import OrderedDict
from datetime import datetime

import regex as re
from pytz import UTC, UnknownTimeZoneError, timezone
from tzlocal import get_localzone

from dateparser.timezone_parser import StaticTzInfo, _tz_offsets


def strip_braces(date_string):
    return re.sub(r"[{}()<>\[\]]+", "", date_string)


def normalize_unicode(string, form="NFKD"):
    return "".join(
        c
        for c in unicodedata.normalize(form, string)
        if unicodedata.category(c) != "Mn"
    )


def combine_dicts(primary_dict, supplementary_dict):
    combined_dict = OrderedDict()
    for key, value in primary_dict.items():
        if key in supplementary_dict:
            if isinstance(value, list):
                combined_dict[key] = value + supplementary_dict[key]
            elif isinstance(value, dict):
                combined_dict[key] = combine_dicts(value, supplementary_dict[key])
            else:
                combined_dict[key] = supplementary_dict[key]
        else:
            combined_dict[key] = primary_dict[key]
    remaining_keys = [
        key for key in supplementary_dict.keys() if key not in primary_dict.keys()
    ]
    for key in remaining_keys:
        combined_dict[key] = supplementary_dict[key]
    return combined_dict


def find_date_separator(format):
    m = re.search(r"(?:(?:%[dbBmaA])(\W))+", format)
    if m:
        return m.group(1)


def _get_missing_parts(fmt):
    """
    Return a list containing missing parts (day, month, year)
    from a date format checking its directives
    """
    directive_mapping = {
        "day": ["%d", "%-d", "%j", "%-j"],
        "month": ["%b", "%B", "%m", "%-m"],
        "year": ["%y", "%-y", "%Y"],
    }

    missing = [
        field
        for field in ("day", "month", "year")
        if not any(directive in fmt for directive in directive_mapping[field])
    ]
    return missing


def get_timezone_from_tz_string(tz_string):
    try:
        return timezone(tz_string)
    except UnknownTimeZoneError as e:
        for name, info in _tz_offsets:
            if info["regex"].search(" %s" % tz_string):
                return StaticTzInfo(name, info["offset"])
        else:
            raise e


def localize_timezone(date_time, tz_string):
    if date_time.tzinfo:
        return date_time

    tz = get_timezone_from_tz_string(tz_string)

    if hasattr(tz, "localize"):
        date_time = tz.localize(date_time)
    else:
        date_time = date_time.replace(tzinfo=tz)

    return date_time


def apply_tzdatabase_timezone(date_time, pytz_string):
    usr_timezone = timezone(pytz_string)

    if date_time.tzinfo != usr_timezone:
        date_time = date_time.astimezone(usr_timezone)

    return date_time


def apply_dateparser_timezone(utc_datetime, offset_or_timezone_abb):
    for name, info in _tz_offsets:
        if info["regex"].search(" %s" % offset_or_timezone_abb):
            tz = StaticTzInfo(name, info["offset"])
            return utc_datetime.astimezone(tz)


def apply_timezone(date_time, tz_string):
    if not date_time.tzinfo:
        if hasattr(UTC, "localize"):
            date_time = UTC.localize(date_time)
        else:
            date_time = date_time.replace(tzinfo=UTC)

    new_datetime = apply_dateparser_timezone(date_time, tz_string)

    if not new_datetime:
        new_datetime = apply_tzdatabase_timezone(date_time, tz_string)

    return new_datetime


def apply_timezone_from_settings(date_obj, settings):
    tz = get_localzone()
    if settings is None:
        return date_obj

    if "local" in settings.TIMEZONE.lower():
        if hasattr(tz, "localize"):
            date_obj = tz.localize(date_obj)
        else:
            date_obj = date_obj.replace(tzinfo=tz)
    else:
        date_obj = localize_timezone(date_obj, settings.TIMEZONE)

    if settings.TO_TIMEZONE:
        date_obj = apply_timezone(date_obj, settings.TO_TIMEZONE)

    if settings.RETURN_AS_TIMEZONE_AWARE is not True:
        date_obj = date_obj.replace(tzinfo=None)

    return date_obj


def get_last_day_of_month(year, month):
    return calendar.monthrange(year, month)[1]


def get_previous_leap_year(year):
    return _get_leap_year(year, future=False)


def get_next_leap_year(year):
    return _get_leap_year(year, future=True)


def _get_leap_year(year, future):
    """
    Iterate through previous or next years until it gets a valid leap year
    This is performed to avoid missing or including centurial leap years
    """
    step = 1 if future else -1
    leap_year = year + step
    while not calendar.isleap(leap_year):
        leap_year += step
    return leap_year


def set_correct_day_from_settings(date_obj, settings, current_day=None):
    """Set correct day attending the `PREFER_DAY_OF_MONTH` setting."""
    options = {
        "first": 1,
        "last": get_last_day_of_month(date_obj.year, date_obj.month),
        "current": current_day or datetime.now().day,
    }

    try:
        return date_obj.replace(day=options[settings.PREFER_DAY_OF_MONTH])
    except ValueError:
        return date_obj.replace(day=options["last"])


def set_correct_month_from_settings(date_obj, settings, current_month=None):
    """Set correct month attending the `PREFER_MONTH_OF_YEAR` setting."""
    options = {"first": 1, "last": 12, "current": current_month or datetime.now().month}

    try:
        return date_obj.replace(month=options[settings.PREFER_MONTH_OF_YEAR])
    except ValueError:
        return date_obj.replace(month=options["last"])


def registry(cls):
    def choose(creator):
        def constructor(cls, *args, **kwargs):
            key = cls.get_key(*args, **kwargs)

            if not hasattr(cls, "__registry_dict"):
                setattr(cls, "__registry_dict", {})
            registry_dict = getattr(cls, "__registry_dict")

            if key not in registry_dict:
                registry_dict[key] = creator(cls, *args)
                setattr(registry_dict[key], "registry_key", key)
            return registry_dict[key]

        return staticmethod(constructor)

    if not (
        hasattr(cls, "get_key")
        and isinstance(cls.get_key, types.MethodType)
        and cls.get_key.__self__ is cls
    ):
        raise NotImplementedError(
            "Registry classes require to implement class method get_key"
        )

    setattr(cls, "__new__", choose(cls.__new__))
    return cls


def get_logger():
    setup_logging()
    return logging.getLogger("dateparser")


def setup_logging():
    if len(logging.root.handlers):
        return

    config = {
        "version": 1,
        "disable_existing_loggers": True,
        "formatters": {
            "console": {
                "format": "%(asctime)s %(levelname)s: [%(name)s] %(message)s",
            },
        },
        "handlers": {
            "console": {
                "level": logging.DEBUG,
                "class": "logging.StreamHandler",
                "formatter": "console",
                "stream": "ext://sys.stdout",
            },
        },
        "root": {
            "level": logging.DEBUG,
            "handlers": ["console"],
        },
    }
    logging.config.dictConfig(config)



info = {
    "name": "nd",
    "date_order": "DMY",
    "january": [
        "zib",
        "zibandlela"
    ],
    "february": [
        "nhlo",
        "nhlolanja"
    ],
    "march": [
        "mbi",
        "mbimbitho"
    ],
    "april": [
        "mab",
        "mabasa"
    ],
    "may": [
        "nkw",
        "nkwenkwezi"
    ],
    "june": [
        "nhla",
        "nhlangula"
    ],
    "july": [
        "ntu",
        "ntulikazi"
    ],
    "august": [
        "ncw",
        "ncwabakazi"
    ],
    "september": [
        "mpan",
        "mpandula"
    ],
    "october": [
        "mfu",
        "mfumfu"
    ],
    "november": [
        "lwe",
        "lwezi"
    ],
    "december": [
        "mpal",
        "mpalakazi"
    ],
    "monday": [
        "mvu",
        "mvulo"
    ],
    "tuesday": [
        "sib",
        "sibili"
    ],
    "wednesday": [
        "sit",
        "sithathu"
    ],
    "thursday": [
        "sin",
        "sine"
    ],
    "friday": [
        "sih",
        "sihlanu"
    ],
    "saturday": [
        "mgq",
        "mgqibelo"
    ],
    "sunday": [
        "son",
        "sonto"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "umnyaka"
    ],
    "month": [
        "inyangacale"
    ],
    "week": [
        "iviki"
    ],
    "day": [
        "ilanga"
    ],
    "hour": [
        "ihola"
    ],
    "minute": [
        "umuzuzu"
    ],
    "second": [
        "isekendi"
    ],
    "relative-type": {
        "0 day ago": [
            "lamuhla"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "izolo"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "kusasa"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from dateparser.conf import apply_settings
from dateparser.search.detection import BaseLanguageDetector
from dateparser.utils import normalize_unicode


class FullTextLanguageDetector(BaseLanguageDetector):
    def __init__(self, languages):
        super(BaseLanguageDetector, self).__init__()
        self.languages = languages[:]
        self.language_unique_chars = []
        self.language_chars = []

    def get_unique_characters(self, settings):
        settings = settings.replace(NORMALIZE=False)

        for language in self.languages:
            chars = language.get_wordchars_for_detection(settings=settings)
            self.language_chars.append(chars)

        for char_set in self.language_chars:
            unique_chars = char_set
            for other_char_set in self.language_chars:
                if other_char_set != char_set:
                    unique_chars = unique_chars - other_char_set
            self.language_unique_chars.append(unique_chars)

    def character_check(self, date_string, settings):
        date_string_set = set(date_string.lower())
        symbol_set = {
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            " ",
            "/",
            "-",
            ")",
            "(",
            ".",
            ":",
            "\\",
            ",",
            "'",
        }
        if date_string_set & symbol_set == date_string_set:
            self.languages = [self.languages[0]]
            return
        self.get_unique_characters(settings=settings)
        for i in range(len(self.languages)):
            for char in self.language_unique_chars[i]:
                if char.lower() in date_string.lower():
                    self.languages = [self.languages[i]]
                    return
        indices_to_pop = []
        for i in range(len(self.languages)):
            if len(date_string_set & self.language_chars[i]) == 0:
                indices_to_pop.append(i)
        self.languages = [
            i for j, i in enumerate(self.languages) if j not in indices_to_pop
        ]

    @apply_settings
    def _best_language(self, date_string, settings=None):
        self.character_check(date_string, settings)
        date_string = normalize_unicode(date_string.lower())
        if len(self.languages) == 1:
            return self.languages[0].shortname
        applicable_languages = []
        for language in self.languages:
            num_words = language.count_applicability(
                date_string, strip_timezone=False, settings=settings
            )
            if num_words[0] > 0 or num_words[1] > 0:
                applicable_languages.append((language.shortname, num_words))
            else:
                num_words = language.count_applicability(
                    date_string, strip_timezone=True, settings=settings
                )
                if num_words[0] > 0 or num_words[1] > 0:
                    applicable_languages.append((language.shortname, num_words))
        if not applicable_languages:
            return None
        return max(applicable_languages, key=lambda p: (p[1][0], p[1][1]))[0]



info = {
    "name": "guz",
    "date_order": "DMY",
    "january": [
        "can",
        "chanuari"
    ],
    "february": [
        "feb",
        "feburari"
    ],
    "march": [
        "mac",
        "machi"
    ],
    "april": [
        "apiriri",
        "apr"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "juni"
    ],
    "july": [
        "chulai",
        "cul"
    ],
    "august": [
        "agosti",
        "agt"
    ],
    "september": [
        "sep",
        "septemba"
    ],
    "october": [
        "okitoba",
        "okt"
    ],
    "november": [
        "nob",
        "nobemba"
    ],
    "december": [
        "dis",
        "disemba"
    ],
    "monday": [
        "chumatato",
        "ctt"
    ],
    "tuesday": [
        "chumaine",
        "cmn"
    ],
    "wednesday": [
        "chumatano",
        "cmt"
    ],
    "thursday": [
        "aramisi",
        "ars"
    ],
    "friday": [
        "ichuma",
        "icm"
    ],
    "saturday": [
        "esabato",
        "est"
    ],
    "sunday": [
        "chumapiri",
        "cpr"
    ],
    "am": [
        "ma",
        "mambia"
    ],
    "pm": [
        "mo",
        "mog"
    ],
    "year": [
        "omwaka"
    ],
    "month": [
        "omotienyi"
    ],
    "week": [
        "omokubio"
    ],
    "day": [
        "rituko"
    ],
    "hour": [
        "ensa"
    ],
    "minute": [
        "edakika"
    ],
    "second": [
        "esekendi"
    ],
    "relative-type": {
        "0 day ago": [
            "rero"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "igoro"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "mambia"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "lt",
    "date_order": "YMD",
    "january": [
        "saus",
        "sausio",
        "sausis"
    ],
    "february": [
        "vas",
        "vasario",
        "vasaris"
    ],
    "march": [
        "kov",
        "kovas",
        "kovo"
    ],
    "april": [
        "bal",
        "balandis",
        "balandžio"
    ],
    "may": [
        "geg",
        "gegužė",
        "gegužės"
    ],
    "june": [
        "birž",
        "birželio",
        "birželis"
    ],
    "july": [
        "liep",
        "liepa",
        "liepos"
    ],
    "august": [
        "rugp",
        "rugpjūtis",
        "rugpjūčio"
    ],
    "september": [
        "rugs",
        "rugsėjis",
        "rugsėjo"
    ],
    "october": [
        "spal",
        "spalio",
        "spalis"
    ],
    "november": [
        "lapkr",
        "lapkritis",
        "lapkričio"
    ],
    "december": [
        "gruod",
        "gruodis",
        "gruodžio"
    ],
    "monday": [
        "pirmadienis",
        "pr"
    ],
    "tuesday": [
        "an",
        "antradienis"
    ],
    "wednesday": [
        "tr",
        "trečiadienis"
    ],
    "thursday": [
        "ketvirtadienis",
        "kt"
    ],
    "friday": [
        "penktadienis",
        "pn"
    ],
    "saturday": [
        "šeštadienis",
        "št"
    ],
    "sunday": [
        "sekmadienis",
        "sk"
    ],
    "am": [
        "priešpiet"
    ],
    "pm": [
        "popiet"
    ],
    "year": [
        "m",
        "metai"
    ],
    "month": [
        "mėn",
        "mėnuo"
    ],
    "week": [
        "sav",
        "savaitė"
    ],
    "day": [
        "d",
        "diena"
    ],
    "hour": [
        "h",
        "val",
        "valanda"
    ],
    "minute": [
        "min",
        "minutė"
    ],
    "second": [
        "s",
        "sek",
        "sekundė"
    ],
    "relative-type": {
        "0 day ago": [
            "šiandien"
        ],
        "0 hour ago": [
            "šią valandą"
        ],
        "0 minute ago": [
            "šią minutę"
        ],
        "0 month ago": [
            "šį mėnesį"
        ],
        "0 second ago": [
            "dabar"
        ],
        "0 week ago": [
            "šią savaitę"
        ],
        "0 year ago": [
            "šiais metais"
        ],
        "1 day ago": [
            "vakar"
        ],
        "1 month ago": [
            "praėjusį mėnesį"
        ],
        "1 week ago": [
            "praėjusią savaitę"
        ],
        "1 year ago": [
            "praėjusiais metais"
        ],
        "in 1 day": [
            "rytoj"
        ],
        "in 1 month": [
            "kitą mėnesį"
        ],
        "in 1 week": [
            "kitą savaitę"
        ],
        "in 1 year": [
            "kitais metais"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "prieš (\\d+[.,]?\\d*) d",
            "prieš (\\d+[.,]?\\d*) dieną",
            "prieš (\\d+[.,]?\\d*) dienų"
        ],
        "\\1 hour ago": [
            "prieš (\\d+[.,]?\\d*) val",
            "prieš (\\d+[.,]?\\d*) valandą",
            "prieš (\\d+[.,]?\\d*) valandų"
        ],
        "\\1 minute ago": [
            "prieš (\\d+[.,]?\\d*) min",
            "prieš (\\d+[.,]?\\d*) minutę",
            "prieš (\\d+[.,]?\\d*) minučių"
        ],
        "\\1 month ago": [
            "prieš (\\d+[.,]?\\d*) mėn",
            "prieš (\\d+[.,]?\\d*) mėnesių",
            "prieš (\\d+[.,]?\\d*) mėnesį"
        ],
        "\\1 second ago": [
            "prieš (\\d+[.,]?\\d*) s",
            "prieš (\\d+[.,]?\\d*) sek",
            "prieš (\\d+[.,]?\\d*) sekundę",
            "prieš (\\d+[.,]?\\d*) sekundžių"
        ],
        "\\1 week ago": [
            "prieš (\\d+[.,]?\\d*) sav",
            "prieš (\\d+[.,]?\\d*) savaitę",
            "prieš (\\d+[.,]?\\d*) savaičių"
        ],
        "\\1 year ago": [
            "prieš (\\d+[.,]?\\d*) m",
            "prieš (\\d+[.,]?\\d*) metus",
            "prieš (\\d+[.,]?\\d*) metų"
        ],
        "in \\1 day": [
            "po (\\d+[.,]?\\d*) d",
            "po (\\d+[.,]?\\d*) dienos",
            "po (\\d+[.,]?\\d*) dienų"
        ],
        "in \\1 hour": [
            "po (\\d+[.,]?\\d*) val",
            "po (\\d+[.,]?\\d*) valandos",
            "po (\\d+[.,]?\\d*) valandų"
        ],
        "in \\1 minute": [
            "po (\\d+[.,]?\\d*) min",
            "po (\\d+[.,]?\\d*) minutės",
            "po (\\d+[.,]?\\d*) minučių"
        ],
        "in \\1 month": [
            "po (\\d+[.,]?\\d*) mėn",
            "po (\\d+[.,]?\\d*) mėnesio",
            "po (\\d+[.,]?\\d*) mėnesių"
        ],
        "in \\1 second": [
            "po (\\d+[.,]?\\d*) s",
            "po (\\d+[.,]?\\d*) sek",
            "po (\\d+[.,]?\\d*) sekundės",
            "po (\\d+[.,]?\\d*) sekundžių"
        ],
        "in \\1 week": [
            "po (\\d+[.,]?\\d*) sav",
            "po (\\d+[.,]?\\d*) savaitės",
            "po (\\d+[.,]?\\d*) savaičių"
        ],
        "in \\1 year": [
            "po (\\d+[.,]?\\d*) m",
            "po (\\d+[.,]?\\d*) metų"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import langdetect

# The below _Factory is set to prevent setting global state of the library
# but still get consistent results.
# Refer : https://github.com/Mimino666/langdetect


class _Factory:
    data = None


def _init_factory():
    if _Factory.data is None:
        _Factory.data = langdetect.detector_factory.DetectorFactory()
        _Factory.data.load_profile(langdetect.detector_factory.PROFILES_DIRECTORY)
        _Factory.data.seed = 0


def _get_language_probablities(text):
    _init_factory()
    detector = _Factory.data.create()
    detector.append(text)
    return detector.get_probabilities()


def detect_languages(text, confidence_threshold):
    language_codes = []
    try:
        parser_data = _get_language_probablities(text)
        for language_candidate in parser_data:
            if language_candidate.prob > confidence_threshold:
                language_codes.append(language_candidate.lang)
    except langdetect.lang_detect_exception.LangDetectException:
        # This exception can be produced with empty strings or inputs without letters like `10-10-2021`.
        # As this could be really common, we ignore them.
        pass
    return language_codes



import collections
from collections.abc import Set
from datetime import datetime, timedelta

import regex as re
from dateutil.relativedelta import relativedelta
from tzlocal import get_localzone

from dateparser.conf import apply_settings, check_settings
from dateparser.custom_language_detection.language_mapping import map_languages
from dateparser.date_parser import date_parser
from dateparser.freshness_date_parser import freshness_date_parser
from dateparser.languages.loader import LocaleDataLoader
from dateparser.parser import _parse_absolute, _parse_nospaces
from dateparser.timezone_parser import pop_tz_offset_from_string
from dateparser.utils import (
    apply_timezone_from_settings,
    get_timezone_from_tz_string,
    set_correct_day_from_settings,
    set_correct_month_from_settings,
)
from dateparser.utils.strptime import strptime as patched_strptime

APOSTROPHE_LOOK_ALIKE_CHARS = [
    "\N{RIGHT SINGLE QUOTATION MARK}",  # '\u2019'
    "\N{MODIFIER LETTER APOSTROPHE}",  # '\u02bc'
    "\N{MODIFIER LETTER TURNED COMMA}",  # '\u02bb'
    "\N{ARMENIAN APOSTROPHE}",  # '\u055a'
    "\N{LATIN SMALL LETTER SALTILLO}",  # '\ua78c'
    "\N{PRIME}",  # '\u2032'
    "\N{REVERSED PRIME}",  # '\u2035'
    "\N{MODIFIER LETTER PRIME}",  # '\u02b9'
    "\N{FULLWIDTH APOSTROPHE}",  # '\uff07'
]

RE_NBSP = re.compile("\xa0", flags=re.UNICODE)
RE_SPACES = re.compile(r"\s+")
RE_TRIM_SPACES = re.compile(r"^\s+(\S.*?)\s+$")
RE_TRIM_COLONS = re.compile(r"(\S.*?):*$")

RE_SANITIZE_SKIP = re.compile(
    r"\t|\n|\r|\u00bb|,\s\u0432\b|\u200e|\xb7|\u200f|\u064e|\u064f", flags=re.M
)
RE_SANITIZE_RUSSIAN = re.compile(r"([\W\d])\u0433\.", flags=re.I | re.U)
RE_SANITIZE_CROATIAN = re.compile(
    r"(\d+)\.\s?(\d+)\.\s?(\d+)\.( u)?", flags=re.I | re.U
)
RE_SANITIZE_PERIOD = re.compile(r"(?<=[^0-9\s])\.", flags=re.U)
RE_SANITIZE_ON = re.compile(r"^.*?on:\s+(.*)")
RE_SANITIZE_APOSTROPHE = re.compile("|".join(APOSTROPHE_LOOK_ALIKE_CHARS))

RE_SEARCH_TIMESTAMP = re.compile(r"^(\d{10})(\d{3})?(\d{3})?(?![^.])")
RE_SEARCH_NEGATIVE_TIMESTAMP = re.compile(r"^([-]\d{10})(\d{3})?(\d{3})?(?![^.])")


def sanitize_spaces(date_string):
    date_string = RE_NBSP.sub(" ", date_string)
    date_string = RE_SPACES.sub(" ", date_string)
    date_string = RE_TRIM_SPACES.sub(r"\1", date_string)
    return date_string


def date_range(begin, end, **kwargs):
    dateutil_error_prone_args = [
        "year",
        "month",
        "week",
        "day",
        "hour",
        "minute",
        "second",
    ]
    for arg in dateutil_error_prone_args:
        if arg in kwargs:
            raise ValueError("Invalid argument: %s" % arg)

    step = relativedelta(**kwargs) if kwargs else relativedelta(days=1)

    date = begin
    while date < end:
        yield date
        date += step

    # handles edge-case when iterating months and last interval is < 30 days
    if kwargs.get("months", 0) > 0 and (date.year, date.month) == (end.year, end.month):
        yield end


def get_intersecting_periods(low, high, period="day"):
    if period not in [
        "year",
        "month",
        "week",
        "day",
        "hour",
        "minute",
        "second",
        "microsecond",
    ]:
        raise ValueError("Invalid period: {}".format(period))

    if high <= low:
        return

    step = relativedelta(**{period + "s": 1})

    current_period_start = low
    if isinstance(current_period_start, datetime):
        reset_arguments = {}
        for test_period in ["microsecond", "second", "minute", "hour"]:
            if test_period == period:
                break
            else:
                reset_arguments[test_period] = 0
        current_period_start = current_period_start.replace(**reset_arguments)

    if period == "week":
        current_period_start = current_period_start - timedelta(
            days=current_period_start.weekday()
        )
    elif period == "month":
        current_period_start = current_period_start.replace(day=1)
    elif period == "year":
        current_period_start = current_period_start.replace(month=1, day=1)

    while current_period_start < high:
        yield current_period_start
        current_period_start += step


def sanitize_date(date_string):
    date_string = RE_SANITIZE_SKIP.sub(" ", date_string)
    date_string = RE_SANITIZE_RUSSIAN.sub(
        r"\1 ", date_string
    )  # remove 'г.' (Russian for year) but not in words
    date_string = RE_SANITIZE_CROATIAN.sub(
        r"\1.\2.\3 ", date_string
    )  # extra '.' and 'u' interferes with parsing relative fractional dates
    date_string = sanitize_spaces(date_string)
    date_string = RE_SANITIZE_PERIOD.sub("", date_string)
    date_string = RE_SANITIZE_ON.sub(r"\1", date_string)
    date_string = RE_TRIM_COLONS.sub(r"\1", date_string)
    date_string = RE_SANITIZE_APOSTROPHE.sub("'", date_string)
    date_string = date_string.strip()
    return date_string


def get_date_from_timestamp(date_string, settings, negative=False):
    if negative:
        match = RE_SEARCH_NEGATIVE_TIMESTAMP.search(date_string)
    else:
        match = RE_SEARCH_TIMESTAMP.search(date_string)

    if match:
        if (
            settings is None
            or settings.TIMEZONE is None
            or "local" in settings.TIMEZONE.lower()
        ):
            # If the timezone in settings is unset, or it's 'local', use the
            # local timezone
            timezone = get_localzone()
        else:
            # Otherwise, use the timezone given in settings
            timezone = get_timezone_from_tz_string(settings.TIMEZONE)

        seconds = int(match.group(1))
        millis = int(match.group(2) or 0)
        micros = int(match.group(3) or 0)
        date_obj = datetime.fromtimestamp(seconds, timezone).replace(
            microsecond=millis * 1000 + micros, tzinfo=None
        )
        date_obj = apply_timezone_from_settings(date_obj, settings)
        return date_obj


def parse_with_formats(date_string, date_formats, settings):
    """Parse with formats and return a dictionary with 'period' and 'obj_date'.

    :returns: :class:`datetime.datetime`, dict or None

    """
    period = "day"
    for date_format in date_formats:
        try:
            date_obj = patched_strptime(date_string, date_format)
        except ValueError:
            continue
        else:
            missing_month = not any(m in date_format for m in ["%m", "%b", "%B"])
            missing_day = "%d" not in date_format
            if missing_month and missing_day:
                period = "year"
                date_obj = set_correct_month_from_settings(date_obj, settings)
                date_obj = set_correct_day_from_settings(date_obj, settings)

            elif missing_month:
                period = "year"
                date_obj = set_correct_month_from_settings(date_obj, settings)

            elif missing_day:
                period = "month"
                date_obj = set_correct_day_from_settings(date_obj, settings)

            if not ("%y" in date_format or "%Y" in date_format):
                today = datetime.today()
                date_obj = date_obj.replace(year=today.year)

            date_obj = apply_timezone_from_settings(date_obj, settings)

            return DateData(date_obj=date_obj, period=period)
    else:
        return DateData(date_obj=None, period=period)


class _DateLocaleParser:
    def __init__(self, locale, date_string, date_formats, settings=None):
        self._settings = settings
        if not (date_formats is None or isinstance(date_formats, (list, tuple, Set))):
            raise TypeError("Date formats should be list, tuple or set of strings")

        self.locale = locale
        self.date_string = date_string
        self.date_formats = date_formats
        self._translated_date = None
        self._translated_date_with_formatting = None
        self._parsers = {
            "timestamp": self._try_timestamp,
            "negative-timestamp": self._try_negative_timestamp,
            "relative-time": self._try_freshness_parser,
            "custom-formats": self._try_given_formats,
            "absolute-time": self._try_absolute_parser,
            "no-spaces-time": self._try_nospaces_parser,
        }

    @classmethod
    def parse(cls, locale, date_string, date_formats=None, settings=None):
        instance = cls(locale, date_string, date_formats, settings)
        return instance._parse()

    def _parse(self):
        for parser_name in self._settings.PARSERS:
            date_data = self._parsers[parser_name]()
            if self._is_valid_date_data(date_data):
                return date_data
        else:
            return None

    def _try_timestamp_parser(self, negative=False):
        return DateData(
            date_obj=get_date_from_timestamp(
                self.date_string, self._settings, negative=negative
            ),
            period="time" if self._settings.RETURN_TIME_AS_PERIOD else "day",
        )

    def _try_timestamp(self):
        return self._try_timestamp_parser()

    def _try_negative_timestamp(self):
        return self._try_timestamp_parser(negative=True)

    def _try_freshness_parser(self):
        try:
            return freshness_date_parser.get_date_data(
                self._get_translated_date(), self._settings
            )
        except (OverflowError, ValueError):
            return None

    def _try_absolute_parser(self):
        return self._try_parser(parse_method=_parse_absolute)

    def _try_nospaces_parser(self):
        return self._try_parser(parse_method=_parse_nospaces)

    def _try_parser(self, parse_method):
        original_order = self._settings.DATE_ORDER

        # Use locale date order unless DATE_ORDER was explicitly set by the caller.
        if (
            self._settings.PREFER_LOCALE_DATE_ORDER
            and "DATE_ORDER" not in self._settings._mod_settings
        ):
            first_order = self.locale.info.get("date_order", original_order)
        else:
            first_order = original_order

        candidates = [first_order]

        # If the caller requires a year (and not a day) and did not set DATE_ORDER,
        # retry once or twice with year-biased orders to resolve month-number ambiguity.
        require_parts = set(getattr(self._settings, "REQUIRE_PARTS", None) or [])
        if (
            "DATE_ORDER" not in self._settings._mod_settings
            and "year" in require_parts
            and "day" not in require_parts
        ):
            for order in ("MYD", "YMD"):
                if order not in candidates:
                    candidates.append(order)

        translated = self._get_translated_date()

        try:
            for order in candidates:
                self._settings.DATE_ORDER = order
                try:
                    date_obj, period = date_parser.parse(
                        translated,
                        parse_method=parse_method,
                        settings=self._settings,
                    )
                    return DateData(date_obj=date_obj, period=period)
                except ValueError:
                    continue
            return None
        finally:
            self._settings.DATE_ORDER = original_order

    def _try_given_formats(self):
        if not self.date_formats:
            return

        return parse_with_formats(
            self._get_translated_date_with_formatting(),
            self.date_formats,
            settings=self._settings,
        )

    def _get_translated_date(self):
        if self._translated_date is None:
            self._translated_date = self.locale.translate(
                self.date_string, keep_formatting=False, settings=self._settings
            )
        return self._translated_date

    def _get_translated_date_with_formatting(self):
        if self._translated_date_with_formatting is None:
            self._translated_date_with_formatting = self.locale.translate(
                self.date_string, keep_formatting=True, settings=self._settings
            )
        return self._translated_date_with_formatting

    def _is_valid_date_data(self, date_data):
        if not isinstance(date_data, DateData):
            return False
        if not date_data["date_obj"] or not date_data["period"]:
            return False
        if date_data["date_obj"] and not isinstance(date_data["date_obj"], datetime):
            return False
        if date_data["period"] not in ("time", "day", "week", "month", "year"):
            return False
        return True


class DateData:
    """
    Class that represents the parsed data with useful information.
    It can be accessed with square brackets like a dict object.
    """

    def __init__(self, *, date_obj=None, period=None, locale=None):
        self.date_obj = date_obj
        self.period = period
        self.locale = locale

    def __getitem__(self, k):
        if not hasattr(self, k):
            raise KeyError(k)
        return getattr(self, k)

    def __setitem__(self, k, v):
        if not hasattr(self, k):
            raise KeyError(k)
        setattr(self, k, v)

    def __repr__(self):
        properties_text = ", ".join(
            "{}={}".format(prop, val.__repr__()) for prop, val in self.__dict__.items()
        )

        return "{}({})".format(self.__class__.__name__, properties_text)


class DateDataParser:
    """
    Class which handles language detection, translation and subsequent generic parsing of
    string representing date and/or time.

    :param languages:
        A list of language codes, e.g. ['en', 'es', 'zh-Hant'].
        If locales are not given, languages and region are
        used to construct locales for translation.
    :type languages: list

    :param locales:
        A list of locale codes, e.g. ['fr-PF', 'qu-EC', 'af-NA'].
        The parser uses only these locales to translate date string.
    :type locales: list

    :param region:
        A region code, e.g. 'IN', '001', 'NE'.
        If locales are not given, languages and region are
        used to construct locales for translation.
    :type region: str

    :param try_previous_locales:
        If True, locales previously used to translate date are tried first.
    :type try_previous_locales: bool

    :param use_given_order:
        If True, locales are tried for translation of date string
        in the order in which they are given.
    :type use_given_order: bool

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :param detect_languages_function:
        A function for language detection that takes as input a `text` and a `confidence_threshold`,
        and returns a list of detected language codes.
        Note: this function is only used if ``languages`` and ``locales`` are not provided.
    :type detect_languages_function: function

    :return: A parser instance

    :raises:
         ``ValueError``: Unknown Language, ``TypeError``: Languages argument must be a list,
         ``SettingValidationError``: A provided setting is not valid.
    """

    locale_loader = None

    @apply_settings
    def __init__(
        self,
        languages=None,
        locales=None,
        region=None,
        try_previous_locales=False,
        use_given_order=False,
        settings=None,
        detect_languages_function=None,
    ):
        if languages is not None and not isinstance(languages, (list, tuple, Set)):
            raise TypeError(
                "languages argument must be a list (%r given)" % type(languages)
            )

        if locales is not None and not isinstance(locales, (list, tuple, Set)):
            raise TypeError(
                "locales argument must be a list (%r given)" % type(locales)
            )

        if region is not None and not isinstance(region, str):
            raise TypeError("region argument must be str (%r given)" % type(region))

        if not isinstance(try_previous_locales, bool):
            raise TypeError(
                "try_previous_locales argument must be a boolean (%r given)"
                % type(try_previous_locales)
            )

        if not isinstance(use_given_order, bool):
            raise TypeError(
                "use_given_order argument must be a boolean (%r given)"
                % type(use_given_order)
            )

        if not locales and not languages and use_given_order:
            raise ValueError(
                "locales or languages must be given if use_given_order is True"
            )

        check_settings(settings)

        self._settings = settings
        self.try_previous_locales = try_previous_locales
        self.use_given_order = use_given_order
        self.languages = list(languages) if languages else None
        self.locales = locales
        self.region = region
        self.detect_languages_function = detect_languages_function
        self.previous_locales = collections.OrderedDict()

    def get_date_data(self, date_string, date_formats=None):
        """
        Parse string representing date and/or time in recognizable localized formats.
        Supports parsing multiple languages and timezones.

        :param date_string:
            A string representing date and/or time in a recognizably valid format.
        :type date_string: str
        :param date_formats:
            A list of format strings using directives as given
            `here <https://docs.python.org/2/library/datetime.html#strftime-and-strptime-behavior>`_.
            The parser applies formats one by one, taking into account the detected languages.
        :type date_formats: list

        :return: a ``DateData`` object.

        :raises: ValueError - Unknown Language

        .. note:: *Period* values can be a 'day' (default), 'week', 'month', 'year', 'time'.

        *Period* represents the granularity of date parsed from the given string.

        In the example below, since no day information is present, the day is assumed to be current
        day ``16`` from *current date* (which is June 16, 2015, at the moment of writing this).
        Hence, the level of precision is ``month``:

            >>> DateDataParser().get_date_data('March 2015')
            DateData(date_obj=datetime.datetime(2015, 3, 16, 0, 0), period='month', locale='en')

        Similarly, for date strings with no day and month information present, level of precision
        is ``year`` and day ``16`` and month ``6`` are from *current_date*.

            >>> DateDataParser().get_date_data('2014')
            DateData(date_obj=datetime.datetime(2014, 6, 16, 0, 0), period='year', locale='en')

        Dates with time zone indications or UTC offsets are returned in UTC time unless
        specified using `Settings <https://dateparser.readthedocs.io/en/latest/settings.html#settings>`__.

            >>> DateDataParser().get_date_data('23 March 2000, 1:21 PM CET')
            DateData(date_obj=datetime.datetime(2000, 3, 23, 13, 21, tzinfo=<StaticTzInfo 'CET'>),
            period='day', locale='en')

        """
        if not isinstance(date_string, str):
            raise TypeError("Input type must be str")

        res = parse_with_formats(date_string, date_formats or [], self._settings)
        if res["date_obj"]:
            return res

        date_string = sanitize_date(date_string)

        for locale in self._get_applicable_locales(date_string):
            parsed_date = _DateLocaleParser.parse(
                locale, date_string, date_formats, settings=self._settings
            )
            if parsed_date:
                parsed_date["locale"] = locale.shortname
                if self.try_previous_locales:
                    self.previous_locales[locale] = None
                return parsed_date
        else:
            return DateData(date_obj=None, period="day", locale=None)

    def get_date_tuple(self, *args, **kwargs):
        date_data = self.get_date_data(*args, **kwargs)
        fields = date_data.__dict__.keys()
        date_tuple = collections.namedtuple("DateData", fields)
        return date_tuple(**date_data.__dict__)

    def _get_applicable_locales(self, date_string):
        pop_tz_cache = []

        def date_strings():
            """A generator instead of a static list to avoid calling
            pop_tz_offset_from_string if the first locale matches on unmodified
            date_string.
            """
            yield date_string
            if not pop_tz_cache:
                stripped_date_string, _ = pop_tz_offset_from_string(
                    date_string, as_offset=False
                )
                if stripped_date_string == date_string:
                    stripped_date_string = None
                pop_tz_cache[:] = [stripped_date_string]
            (stripped_date_string,) = pop_tz_cache
            if stripped_date_string is not None:
                yield stripped_date_string

        if self.try_previous_locales:
            for locale in self.previous_locales.keys():
                for s in date_strings():
                    if self._is_applicable_locale(locale, s):
                        yield locale

        if self.detect_languages_function and not self.languages and not self.locales:
            detected_languages = self.detect_languages_function(
                text=date_string,
                confidence_threshold=self._settings.LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD,
            )

            self.languages = map_languages(detected_languages)

        for locale in self._get_locale_loader().get_locales(
            languages=self.languages,
            locales=self.locales,
            region=self.region,
            use_given_order=self.use_given_order,
        ):
            for s in date_strings():
                if self._is_applicable_locale(locale, s):
                    yield locale

        if self._settings.DEFAULT_LANGUAGES:
            for locale in self._get_locale_loader().get_locales(
                languages=self._settings.DEFAULT_LANGUAGES,
                locales=None,
                region=self.region,
                use_given_order=self.use_given_order,
            ):
                yield locale

    def _is_applicable_locale(self, locale, date_string):
        return locale.is_applicable(
            date_string,
            strip_timezone=False,  # it is stripped outside
            settings=self._settings,
        )

    @classmethod
    def _get_locale_loader(cls):
        if not cls.locale_loader:
            cls.locale_loader = LocaleDataLoader()
        return cls.locale_loader



info = {
    "name": "ug",
    "date_order": "YMD",
    "january": [
        "يانۋار"
    ],
    "february": [
        "فېۋرال"
    ],
    "march": [
        "مارت"
    ],
    "april": [
        "ئاپرېل"
    ],
    "may": [
        "ماي"
    ],
    "june": [
        "ئىيۇن"
    ],
    "july": [
        "ئىيۇل"
    ],
    "august": [
        "ئاۋغۇست"
    ],
    "september": [
        "سېنتەبىر"
    ],
    "october": [
        "ئۆكتەبىر"
    ],
    "november": [
        "نويابىر"
    ],
    "december": [
        "دېكابىر"
    ],
    "monday": [
        "دۈ",
        "دۈشەنبە"
    ],
    "tuesday": [
        "سە",
        "سەيشەنبە"
    ],
    "wednesday": [
        "چا",
        "چارشەنبە"
    ],
    "thursday": [
        "پە",
        "پەيشەنبە"
    ],
    "friday": [
        "جۈ",
        "جۈمە"
    ],
    "saturday": [
        "شە",
        "شەنبە"
    ],
    "sunday": [
        "يە",
        "يەكشەنبە"
    ],
    "am": [
        "چب",
        "چۈشتىن بۇرۇن"
    ],
    "pm": [
        "چك",
        "چۈشتىن كېيىن"
    ],
    "year": [
        "يىل"
    ],
    "month": [
        "ئاي"
    ],
    "week": [
        "ھەپتە"
    ],
    "day": [
        "كۈن"
    ],
    "hour": [
        "سائەت"
    ],
    "minute": [
        "مىنۇت"
    ],
    "second": [
        "سېكۇنت"
    ],
    "relative-type": {
        "0 day ago": [
            "بۈگۈن"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "بۇ ئاي"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "بۇ ھەپتە"
        ],
        "0 year ago": [
            "بۇ يىل"
        ],
        "1 day ago": [
            "تۈنۈگۈن"
        ],
        "1 month ago": [
            "ئۆتكەن ئاي"
        ],
        "1 week ago": [
            "ئۆتكەن ھەپتە"
        ],
        "1 year ago": [
            "ئۆتكەن يىل"
        ],
        "in 1 day": [
            "ئەتە"
        ],
        "in 1 month": [
            "كېلەر ئاي"
        ],
        "in 1 week": [
            "كېلەر ھەپتە"
        ],
        "in 1 year": [
            "كېلەر يىل"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) كۈن ئىلگىرى"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) سائەت ئىلگىرى"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) مىنۇت ئىلگىرى"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) ئاي ئىلگىرى"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) سېكۇنت ئىلگىرى"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) ھەپتە ئىلگىرى"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) يىل ئىلگىرى"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) كۈندىن كېيىن"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) سائەتتىن كېيىن"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) مىنۇتتىن كېيىن"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) ئايدىن كېيىن"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) سېكۇنتتىن كېيىن"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) ھەپتىدىن كېيىن"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) يىلدىن كېيىن"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "nus",
    "date_order": "DMY",
    "january": [
        "tiop",
        "tiop thar pɛt"
    ],
    "february": [
        "pɛt"
    ],
    "march": [
        "duɔ̱ɔ̱",
        "duɔ̱ɔ̱ŋ"
    ],
    "april": [
        "guak"
    ],
    "may": [
        "duä",
        "duät"
    ],
    "june": [
        "kor",
        "kornyoot"
    ],
    "july": [
        "pay",
        "pay yie̱tni"
    ],
    "august": [
        "thoo",
        "tho̱o̱r"
    ],
    "september": [
        "tɛɛ",
        "tɛɛr"
    ],
    "october": [
        "laa",
        "laath"
    ],
    "november": [
        "kur"
    ],
    "december": [
        "tid",
        "tio̱p in di̱i̱t"
    ],
    "monday": [
        "jiec",
        "jiec la̱t"
    ],
    "tuesday": [
        "rɛw",
        "rɛw lätni"
    ],
    "wednesday": [
        "diɔ̱k",
        "diɔ̱k lätni"
    ],
    "thursday": [
        "ŋuaan",
        "ŋuaan lätni"
    ],
    "friday": [
        "dhieec",
        "dhieec lätni"
    ],
    "saturday": [
        "bäkɛl",
        "bäkɛl lätni"
    ],
    "sunday": [
        "cäŋ",
        "cäŋ kuɔth"
    ],
    "am": [
        "rw"
    ],
    "pm": [
        "tŋ"
    ],
    "year": [
        "ruɔ̱n"
    ],
    "month": [
        "pay"
    ],
    "week": [
        "jiɔk"
    ],
    "day": [
        "cäŋ"
    ],
    "hour": [
        "thaak"
    ],
    "minute": [
        "minit"
    ],
    "second": [
        "thɛkɛni"
    ],
    "relative-type": {
        "0 day ago": [
            "walɛ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "pan"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ruun"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



default_parsers = [
    "timestamp",
    "relative-time",
    "custom-formats",
    "absolute-time",
]

settings = {
    # Date order
    "DATE_ORDER": "MDY",
    "PREFER_LOCALE_DATE_ORDER": True,
    # Timezone related
    "TIMEZONE": "local",
    "TO_TIMEZONE": False,
    "RETURN_AS_TIMEZONE_AWARE": "default",
    # Incomplete dates
    "PREFER_DAY_OF_MONTH": "current",
    "PREFER_MONTH_OF_YEAR": "current",
    "PREFER_DATES_FROM": "current_period",
    "RELATIVE_BASE": False,
    "STRICT_PARSING": False,
    "REQUIRE_PARTS": [],
    # Language detection
    "SKIP_TOKENS": ["t"],
    "NORMALIZE": True,
    "DEFAULT_LANGUAGES": [],
    # Optional language detection
    "LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD": 0.5,
    # Time span settings
    "RETURN_TIME_SPAN": False,
    "DEFAULT_START_OF_WEEK": "monday",
    "DEFAULT_DAYS_IN_MONTH": 30,
    # Other settings
    "RETURN_TIME_AS_PERIOD": False,
    "PARSERS": default_parsers,
    "CACHE_SIZE_LIMIT": 1000,
}



__version__ = "1.4.0"

from .conf import apply_settings
from .date import DateDataParser

_default_parser = DateDataParser()


@apply_settings
def parse(
    date_string,
    date_formats=None,
    languages=None,
    locales=None,
    region=None,
    settings=None,
    detect_languages_function=None,
):
    """Parse date and time from given date string.

    :param date_string:
        A string representing date and/or time in a recognizably valid format.
    :type date_string: str

    :param date_formats:
        A list of format strings using directives as given
        `here <https://docs.python.org/2/library/datetime.html#strftime-and-strptime-behavior>`_.
        The parser applies formats one by one, taking into account the detected languages/locales.
    :type date_formats: list

    :param languages:
        A list of language codes, e.g. ['en', 'es', 'zh-Hant'].
        If locales are not given, languages and region are used to construct locales for translation.
    :type languages: list

    :param locales:
        A list of locale codes, e.g. ['fr-PF', 'qu-EC', 'af-NA'].
        The parser uses only these locales to translate date string.
    :type locales: list

    :param region:
        A region code, e.g. 'IN', '001', 'NE'.
        If locales are not given, languages and region are used to construct locales for translation.
    :type region: str

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :param detect_languages_function:
        A function for language detection that takes as input a string (the `date_string`) and
        a `confidence_threshold`, and returns a list of detected language codes.
        Note: this function is only used if ``languages`` and ``locales`` are not provided.
    :type detect_languages_function: function

    :return: Returns :class:`datetime <datetime.datetime>` representing parsed date if successful, else returns None
    :rtype: :class:`datetime <datetime.datetime>`.
    :raises:
        ``ValueError``: Unknown Language, ``TypeError``: Languages argument must be a list,
        ``SettingValidationError``: A provided setting is not valid.
    """
    parser = _default_parser

    if (
        languages
        or locales
        or region
        or detect_languages_function
        or not settings._default
    ):
        parser = DateDataParser(
            languages=languages,
            locales=locales,
            region=region,
            settings=settings,
            detect_languages_function=detect_languages_function,
        )

    data = parser.get_date_data(date_string, date_formats)

    if data:
        return data["date_obj"]



import os
import shutil
from collections import OrderedDict

from git import Repo


def get_raw_data():
    cldr_version = "31.0.1"
    raw_data_directory = "../raw_data"

    cldr_data = {
        "dates_full": {
            "url": "https://github.com/unicode-cldr/cldr-dates-full.git",
            "dir": "{}/cldr_dates_full/".format(raw_data_directory),
        },
        "core": {
            "url": "https://github.com/unicode-cldr/cldr-core.git",
            "dir": "{}/cldr_core/".format(raw_data_directory),
        },
        "rbnf": {
            "url": "https://github.com/unicode-cldr/cldr-rbnf.git",
            "dir": "{}/cldr_rbnf/".format(raw_data_directory),
        },
    }

    if os.path.isdir(raw_data_directory):
        # remove current raw data
        shutil.rmtree(raw_data_directory)
    os.mkdir(raw_data_directory)

    for name, data in cldr_data.items():
        print('Clonning "{}" from: {}'.format(name, data["url"]))
        repo = Repo.clone_from(data["url"], data["dir"], branch="master")
        repo.git.co(cldr_version)


def get_dict_difference(parent_dict, child_dict):
    difference_dict = OrderedDict()
    for key, child_value in child_dict.items():
        parent_value = parent_dict.get(key)
        child_specific_value = None
        if not parent_value:
            child_specific_value = child_value
        elif isinstance(child_value, list):
            child_specific_value = sorted(set(child_value) - set(parent_value))
        elif isinstance(child_value, dict):
            child_specific_value = get_dict_difference(parent_value, child_value)
        elif child_value != parent_value:
            child_specific_value = child_value
        if child_specific_value:
            difference_dict[key] = child_specific_value
    return difference_dict


def combine_dicts(primary_dict, supplementary_dict):
    combined_dict = OrderedDict()
    for key, value in primary_dict.items():
        if key in supplementary_dict:
            if isinstance(value, list):
                combined_dict[key] = value + supplementary_dict[key]
            elif isinstance(value, dict):
                combined_dict[key] = combine_dicts(value, supplementary_dict[key])
            else:
                combined_dict[key] = supplementary_dict[key]
        else:
            combined_dict[key] = primary_dict[key]
    remaining_keys = [
        key for key in supplementary_dict.keys() if key not in primary_dict.keys()
    ]
    for key in remaining_keys:
        combined_dict[key] = supplementary_dict[key]
    return combined_dict



info = {
    "name": "kea",
    "date_order": "DMY",
    "january": [
        "jan",
        "janeru"
    ],
    "february": [
        "feb",
        "febreru"
    ],
    "march": [
        "mar",
        "marsu"
    ],
    "april": [
        "abr",
        "abril"
    ],
    "may": [
        "mai",
        "maiu"
    ],
    "june": [
        "jun",
        "junhu"
    ],
    "july": [
        "jul",
        "julhu"
    ],
    "august": [
        "ago",
        "agostu"
    ],
    "september": [
        "set",
        "setenbru"
    ],
    "october": [
        "otu",
        "otubru"
    ],
    "november": [
        "nuv",
        "nuvenbru"
    ],
    "december": [
        "diz",
        "dizenbru"
    ],
    "monday": [
        "sig",
        "sigunda-fera"
    ],
    "tuesday": [
        "ter",
        "tersa-fera"
    ],
    "wednesday": [
        "kua",
        "kuarta-fera"
    ],
    "thursday": [
        "kin",
        "kinta-fera"
    ],
    "friday": [
        "ses",
        "sesta-fera"
    ],
    "saturday": [
        "sab",
        "sabadu",
        "sábadu"
    ],
    "sunday": [
        "dum",
        "dumingu"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "anu"
    ],
    "month": [
        "mes"
    ],
    "week": [
        "sim",
        "simana"
    ],
    "day": [
        "dia"
    ],
    "hour": [
        "h",
        "ora"
    ],
    "minute": [
        "m",
        "min",
        "minutu"
    ],
    "second": [
        "s",
        "sig",
        "sigundu"
    ],
    "relative-type": {
        "0 day ago": [
            "oji"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "es mes li"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "es simana li"
        ],
        "0 year ago": [
            "es anu li"
        ],
        "1 day ago": [
            "onti"
        ],
        "1 month ago": [
            "mes pasadu"
        ],
        "1 week ago": [
            "simana pasadu"
        ],
        "1 year ago": [
            "anu pasadu"
        ],
        "in 1 day": [
            "manha"
        ],
        "in 1 month": [
            "prósimu mes"
        ],
        "in 1 week": [
            "prósimu simana"
        ],
        "in 1 year": [
            "prósimu anu"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "a ten (\\d+[.,]?\\d*) dia"
        ],
        "\\1 hour ago": [
            "a ten (\\d+[.,]?\\d*) ora"
        ],
        "\\1 minute ago": [
            "a ten (\\d+[.,]?\\d*) m",
            "a ten (\\d+[.,]?\\d*) min",
            "a ten (\\d+[.,]?\\d*) minutu"
        ],
        "\\1 month ago": [
            "a ten (\\d+[.,]?\\d*) mes"
        ],
        "\\1 second ago": [
            "a ten (\\d+[.,]?\\d*) s",
            "a ten (\\d+[.,]?\\d*) sig",
            "a ten (\\d+[.,]?\\d*) sigundu"
        ],
        "\\1 week ago": [
            "a ten (\\d+[.,]?\\d*) sim",
            "a ten (\\d+[.,]?\\d*) simana"
        ],
        "\\1 year ago": [
            "a ten (\\d+[.,]?\\d*) anu"
        ],
        "in \\1 day": [
            "di li (\\d+[.,]?\\d*) dia"
        ],
        "in \\1 hour": [
            "di li (\\d+[.,]?\\d*) ora"
        ],
        "in \\1 minute": [
            "di li (\\d+[.,]?\\d*) m",
            "di li (\\d+[.,]?\\d*) min",
            "di li (\\d+[.,]?\\d*) minutu"
        ],
        "in \\1 month": [
            "di li (\\d+[.,]?\\d*) mes"
        ],
        "in \\1 second": [
            "di li (\\d+[.,]?\\d*) s",
            "di li (\\d+[.,]?\\d*) sig",
            "di li (\\d+[.,]?\\d*) sigundu"
        ],
        "in \\1 week": [
            "di li (\\d+[.,]?\\d*) sim",
            "di li (\\d+[.,]?\\d*) simana"
        ],
        "in \\1 year": [
            "di li (\\d+[.,]?\\d*) anu"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "mfe",
    "date_order": "DMY",
    "january": [
        "zan",
        "zanvie"
    ],
    "february": [
        "fev",
        "fevriye"
    ],
    "march": [
        "mar",
        "mars"
    ],
    "april": [
        "avr",
        "avril"
    ],
    "may": [
        "me"
    ],
    "june": [
        "zin"
    ],
    "july": [
        "zil",
        "zilye"
    ],
    "august": [
        "out"
    ],
    "september": [
        "sep",
        "septam"
    ],
    "october": [
        "okt",
        "oktob"
    ],
    "november": [
        "nov",
        "novam"
    ],
    "december": [
        "des",
        "desam"
    ],
    "monday": [
        "lin",
        "lindi"
    ],
    "tuesday": [
        "mar",
        "mardi"
    ],
    "wednesday": [
        "mer",
        "merkredi"
    ],
    "thursday": [
        "ze",
        "zedi"
    ],
    "friday": [
        "van",
        "vandredi"
    ],
    "saturday": [
        "sam",
        "samdi"
    ],
    "sunday": [
        "dim",
        "dimans"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "lane"
    ],
    "month": [
        "mwa"
    ],
    "week": [
        "semenn"
    ],
    "day": [
        "zour"
    ],
    "hour": [
        "ler"
    ],
    "minute": [
        "minit"
    ],
    "second": [
        "segonn"
    ],
    "relative-type": {
        "0 day ago": [
            "zordi"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yer"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "demin"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from dateparser.search.search import DateSearchWithDetection

_search_with_detection = DateSearchWithDetection()


def search_dates(
    text,
    languages=None,
    settings=None,
    add_detected_language=False,
    detect_languages_function=None,
):
    """Find all substrings of the given string which represent date and/or time and parse them.

    :param text:
        A string in a natural language which may contain date and/or time expressions.
    :type text: str

    :param languages:
        A list of two letters language codes.e.g. ['en', 'es']. If languages are given, it will
        not attempt to detect the language.
    :type languages: list

    :param settings:
        Configure customized behavior using settings defined in :mod:`dateparser.conf.Settings`.
    :type settings: dict

    :param add_detected_language:
        Indicates if we want the detected language returned in the tuple.
    :type add_detected_language: bool

    :param detect_languages_function:
        A function for language detection that takes as input a `text` and a `confidence_threshold`,
        and returns a list of detected language codes.
        Note: detect_languages_function is only uses if `languages` are not provided.
    :type detect_languages_function: function

    :return: Returns list of tuples containing:
        substrings representing date and/or time, corresponding :mod:`datetime.datetime`
        object and detected language if *add_detected_language* is True.
        Returns None if no dates that can be parsed are found.
    :rtype: list
    :raises: ValueError - Unknown Language

    >>> from dateparser.search import search_dates
    >>> search_dates('The first artificial Earth satellite was launched on 4 October 1957.')
    [('on 4 October 1957', datetime.datetime(1957, 10, 4, 0, 0))]

    >>> search_dates('The first artificial Earth satellite was launched on 4 October 1957.',
    >>>              add_detected_language=True)
    [('on 4 October 1957', datetime.datetime(1957, 10, 4, 0, 0), 'en')]

    >>> search_dates("The client arrived to the office for the first time in March 3rd, 2004 "
    >>>              "and got serviced, after a couple of months, on May 6th 2004, the customer "
    >>>              "returned indicating a defect on the part")
    [('in March 3rd, 2004 and', datetime.datetime(2004, 3, 3, 0, 0)),
     ('on May 6th 2004', datetime.datetime(2004, 5, 6, 0, 0))]

    """
    text = _search_with_detection.preprocess_text(text, languages)

    result = _search_with_detection.search_dates(
        text=text,
        languages=languages,
        settings=settings,
        detect_languages_function=detect_languages_function,
    )
    dates = result.get("Dates")
    if dates:
        if add_detected_language:
            language = result.get("Language")
            dates = [date + (language,) for date in dates]
        return dates



import json
import os
import shutil
from collections import OrderedDict

import regex as re

from dateparser_scripts.order_languages import _get_language_locale_dict
from dateparser_scripts.utils import get_dict_difference, get_raw_data

APOSTROPHE_LOOK_ALIKE_CHARS = [
    "\N{RIGHT SINGLE QUOTATION MARK}",  # '\u2019'
    "\N{MODIFIER LETTER APOSTROPHE}",  # '\u02bc'
    "\N{MODIFIER LETTER TURNED COMMA}",  # '\u02bb'
    "\N{ARMENIAN APOSTROPHE}",  # '\u055a'
    "\N{LATIN SMALL LETTER SALTILLO}",  # '\ua78c'
    "\N{PRIME}",  # '\u2032'
    "\N{REVERSED PRIME}",  # '\u2035'
    "\N{MODIFIER LETTER PRIME}",  # '\u02b9'
    "\N{FULLWIDTH APOSTROPHE}",  # '\uff07'
]

DATE_ORDER_PATTERN = re.compile(
    "([DMY])+\u200f*[-/. \t]*([DMY])+\u200f*[-/. \t]*([DMY])+"
)
RELATIVE_PATTERN = re.compile(r"(?<![\+\-]\s*)\{0\}")
DEFAULT_MONTH_PATTERN = re.compile(r"^M?\d+$", re.U)
RE_SANITIZE_APOSTROPHE = re.compile("|".join(APOSTROPHE_LOOK_ALIKE_CHARS))
AM_PATTERN = re.compile(r"^\s*[Aa]\s*\.?\s*[Mm]\s*\.?\s*$")
PM_PATTERN = re.compile(r"^\s*[Pp]\s*\.?\s*[Mm]\s*\.?\s*$")
PARENTHESIS_PATTERN = re.compile(r"[\(\)]")

cldr_dates_full_dir = "../raw_data/cldr_dates_full/main/"


def _filter_relative_string(relative_string):
    return (
        isinstance(relative_string, str)
        and RELATIVE_PATTERN.search(relative_string)
        and not PARENTHESIS_PATTERN.search(relative_string)
    )


def _filter_month_name(month_name):
    return not DEFAULT_MONTH_PATTERN.match(month_name)


def _retrieve_locale_data(locale):
    ca_gregorian_file = cldr_dates_full_dir + locale + "/ca-gregorian.json"
    dateFields_file = cldr_dates_full_dir + locale + "/dateFields.json"
    with open(ca_gregorian_file) as f:
        cldr_gregorian_data = json.load(f, object_pairs_hook=OrderedDict)

    with open(dateFields_file) as g:
        cldr_datefields_data = json.load(g, object_pairs_hook=OrderedDict)

    gregorian_dict = cldr_gregorian_data["main"][locale]["dates"]["calendars"][
        "gregorian"
    ]
    date_fields_dict = cldr_datefields_data["main"][locale]["dates"]["fields"]

    json_dict = OrderedDict()

    field_keys_1 = ["stand-alone", "format"]
    field_keys_2 = [
        "wide",
        "abbreviated",
    ]  # neglecting "narrow" to avoid problems in translation
    year_keys = ["year", "year-short", "year-narrow"]
    month_keys = ["month", "month-short", "month-narrow"]
    week_keys = ["week", "week-short", "week-narrow"]
    day_keys = ["day", "day-short", "day-narrow"]
    hour_keys = ["hour", "hour-short", "hour-narrow"]
    minute_keys = ["minute", "minute-short", "minute-narrow"]
    second_keys = ["second", "second-short", "second-narrow"]
    relative_keys = ["relativeTimePattern-count-one", "relativeTimePattern-count-other"]

    json_dict["name"] = locale

    try:
        date_format_string = gregorian_dict["dateFormats"]["short"].upper()
    except AttributeError:
        date_format_string = gregorian_dict["dateFormats"]["short"]["_value"].upper()

    json_dict["date_order"] = DATE_ORDER_PATTERN.sub(
        r"\1\2\3", DATE_ORDER_PATTERN.search(date_format_string).group()
    )

    json_dict["january"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["1"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["february"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["2"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["march"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["3"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["april"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["4"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["may"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["5"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["june"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["6"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["july"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["7"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["august"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["8"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["september"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["9"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["october"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["10"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["november"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["11"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["december"] = list(
        filter(
            _filter_month_name,
            [
                gregorian_dict["months"][key1][key2]["12"]
                for key1 in field_keys_1
                for key2 in field_keys_2
            ],
        )
    )

    json_dict["monday"] = [
        gregorian_dict["days"][key1][key2]["mon"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["tuesday"] = [
        gregorian_dict["days"][key1][key2]["tue"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["wednesday"] = [
        gregorian_dict["days"][key1][key2]["wed"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["thursday"] = [
        gregorian_dict["days"][key1][key2]["thu"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["friday"] = [
        gregorian_dict["days"][key1][key2]["fri"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["saturday"] = [
        gregorian_dict["days"][key1][key2]["sat"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["sunday"] = [
        gregorian_dict["days"][key1][key2]["sun"]
        for key1 in field_keys_1
        for key2 in field_keys_2
    ]

    json_dict["am"] = [
        AM_PATTERN.sub("am", x)
        for x in [
            gregorian_dict["dayPeriods"][key1][key2]["am"]
            for key1 in field_keys_1
            for key2 in field_keys_2
        ]
    ]

    json_dict["pm"] = [
        PM_PATTERN.sub("pm", x)
        for x in [
            gregorian_dict["dayPeriods"][key1][key2]["pm"]
            for key1 in field_keys_1
            for key2 in field_keys_2
        ]
    ]

    json_dict["year"] = [date_fields_dict[key]["displayName"] for key in year_keys]

    json_dict["month"] = [date_fields_dict[key]["displayName"] for key in month_keys]

    json_dict["week"] = [date_fields_dict[key]["displayName"] for key in week_keys]

    json_dict["day"] = [date_fields_dict[key]["displayName"] for key in day_keys]

    json_dict["hour"] = [date_fields_dict[key]["displayName"] for key in hour_keys]

    json_dict["minute"] = [date_fields_dict[key]["displayName"] for key in minute_keys]

    json_dict["second"] = [date_fields_dict[key]["displayName"] for key in second_keys]

    json_dict["relative-type"] = OrderedDict()

    json_dict["relative-type"]["1 year ago"] = [
        date_fields_dict[key]["relative-type--1"] for key in year_keys
    ]

    json_dict["relative-type"]["0 year ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in year_keys
    ]

    json_dict["relative-type"]["in 1 year"] = [
        date_fields_dict[key]["relative-type-1"] for key in year_keys
    ]

    json_dict["relative-type"]["1 month ago"] = [
        date_fields_dict[key]["relative-type--1"] for key in month_keys
    ]

    json_dict["relative-type"]["0 month ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in month_keys
    ]

    json_dict["relative-type"]["in 1 month"] = [
        date_fields_dict[key]["relative-type-1"] for key in month_keys
    ]

    json_dict["relative-type"]["1 week ago"] = [
        date_fields_dict[key]["relative-type--1"] for key in week_keys
    ]

    json_dict["relative-type"]["0 week ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in week_keys
    ]

    json_dict["relative-type"]["in 1 week"] = [
        date_fields_dict[key]["relative-type-1"] for key in week_keys
    ]

    json_dict["relative-type"]["1 day ago"] = [
        date_fields_dict[key]["relative-type--1"] for key in day_keys
    ]

    json_dict["relative-type"]["0 day ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in day_keys
    ]

    json_dict["relative-type"]["in 1 day"] = [
        date_fields_dict[key]["relative-type-1"] for key in day_keys
    ]

    json_dict["relative-type"]["0 hour ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in hour_keys
    ]

    json_dict["relative-type"]["0 minute ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in minute_keys
    ]

    json_dict["relative-type"]["0 second ago"] = [
        date_fields_dict[key]["relative-type-0"] for key in second_keys
    ]

    json_dict["relative-type-regex"] = OrderedDict()

    json_dict["relative-type-regex"]["in \\1 year"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in year_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 year ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in year_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 month"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in month_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 month ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in month_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 week"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in week_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 week ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in week_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 day"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in day_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 day ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in day_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 hour"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in hour_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 hour ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in hour_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 minute"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in minute_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 minute ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in minute_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["in \\1 second"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-future"].get(key2)
                for key1 in second_keys
                for key2 in relative_keys
            ],
        )
    )

    json_dict["relative-type-regex"]["\\1 second ago"] = list(
        filter(
            _filter_relative_string,
            [
                date_fields_dict[key1]["relativeTime-type-past"].get(key2)
                for key1 in second_keys
                for key2 in relative_keys
            ],
        )
    )

    return json_dict


def _clean_string(given_string):
    given_string = RE_SANITIZE_APOSTROPHE.sub("'", given_string)
    given_string = given_string.replace(".", "")
    given_string = given_string.lower()
    return " ".join(given_string.split())


def _clean_dict(json_dict):
    """Remove duplicates and sort"""
    for key, value in json_dict.items():
        if isinstance(value, list):
            json_dict[key] = sorted(OrderedDict.fromkeys(map(_clean_string, value)))
        elif isinstance(value, dict):
            json_dict[key] = OrderedDict(sorted(value.items()))
            json_dict[key] = _clean_dict(json_dict[key])
    return OrderedDict(filter(lambda x: x[1], json_dict.items()))


def main():
    get_raw_data()
    language_locale_dict = _get_language_locale_dict()
    parent_directory = "../dateparser_data/cldr_language_data"
    directory = "../dateparser_data/cldr_language_data/date_translation_data/"
    if not os.path.isdir(parent_directory):
        os.mkdir(parent_directory)
    if os.path.isdir(directory):
        shutil.rmtree(directory)
    os.mkdir(directory)

    for language in language_locale_dict:
        json_language_dict = _clean_dict(_retrieve_locale_data(language))
        locale_specific_dict = OrderedDict()
        locales_list = language_locale_dict[language]
        for locale in locales_list:
            json_locale_dict = _clean_dict(_retrieve_locale_data(locale))
            locale_specific_dict[locale] = _clean_dict(
                get_dict_difference(json_language_dict, json_locale_dict)
            )
        json_language_dict["locale_specific"] = OrderedDict(
            sorted(locale_specific_dict.items())
        )
        filename = directory + language + ".json"
        print("writing " + filename)
        json_string = json.dumps(
            json_language_dict, indent=4, separators=(",", ": "), ensure_ascii=False
        ).encode("utf-8")
        with open(filename, "wb") as f:
            f.write(json_string)


if __name__ == "__main__":
    main()



info = {
    "name": "gv",
    "date_order": "YMD",
    "january": [
        "j-guer",
        "jerrey-geuree"
    ],
    "february": [
        "t-arree",
        "toshiaght-arree"
    ],
    "march": [
        "mayrnt"
    ],
    "april": [
        "averil",
        "avrril"
    ],
    "may": [
        "boaldyn"
    ],
    "june": [
        "m-souree",
        "mean-souree"
    ],
    "july": [
        "j-souree",
        "jerrey-souree"
    ],
    "august": [
        "luanistyn"
    ],
    "september": [
        "m-fouyir",
        "mean-fouyir"
    ],
    "october": [
        "j-fouyir",
        "jerrey-fouyir"
    ],
    "november": [
        "m-houney",
        "mee houney"
    ],
    "december": [
        "m-nollick",
        "mee ny nollick"
    ],
    "monday": [
        "jel",
        "jelhein"
    ],
    "tuesday": [
        "jem",
        "jemayrt"
    ],
    "wednesday": [
        "jerc",
        "jercean"
    ],
    "thursday": [
        "jerd",
        "jerdein"
    ],
    "friday": [
        "jeh",
        "jeheiney"
    ],
    "saturday": [
        "jes",
        "jesarn"
    ],
    "sunday": [
        "jed",
        "jedoonee"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "dyo",
    "date_order": "DMY",
    "january": [
        "sa",
        "sanvie"
    ],
    "february": [
        "fe",
        "fébirie"
    ],
    "march": [
        "ma",
        "mars"
    ],
    "april": [
        "ab",
        "aburil"
    ],
    "may": [
        "me",
        "mee"
    ],
    "june": [
        "su",
        "sueŋ"
    ],
    "july": [
        "sú",
        "súuyee"
    ],
    "august": [
        "ut"
    ],
    "september": [
        "se",
        "settembar"
    ],
    "october": [
        "ok",
        "oktobar"
    ],
    "november": [
        "no",
        "novembar"
    ],
    "december": [
        "de",
        "disambar"
    ],
    "monday": [
        "ten",
        "teneŋ"
    ],
    "tuesday": [
        "tal",
        "talata"
    ],
    "wednesday": [
        "ala",
        "alarbay"
    ],
    "thursday": [
        "ara",
        "aramisay"
    ],
    "friday": [
        "arj",
        "arjuma"
    ],
    "saturday": [
        "sib",
        "sibiti"
    ],
    "sunday": [
        "dim",
        "dimas"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "emit"
    ],
    "month": [
        "fuleeŋ"
    ],
    "week": [
        "lóokuŋ"
    ],
    "day": [
        "funak"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "jaat"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "fucen"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "kajom"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "ln",
    "date_order": "DMY",
    "january": [
        "sánzá ya yambo",
        "yan"
    ],
    "february": [
        "fbl",
        "sánzá ya míbalé"
    ],
    "march": [
        "msi",
        "sánzá ya mísáto"
    ],
    "april": [
        "apl",
        "sánzá ya mínei"
    ],
    "may": [
        "mai",
        "sánzá ya mítáno"
    ],
    "june": [
        "sánzá ya motóbá",
        "yun"
    ],
    "july": [
        "sánzá ya nsambo",
        "yul"
    ],
    "august": [
        "agt",
        "sánzá ya mwambe"
    ],
    "september": [
        "stb",
        "sánzá ya libwa"
    ],
    "october": [
        "sánzá ya zómi",
        "ɔtb"
    ],
    "november": [
        "nvb",
        "sánzá ya zómi na mɔ̌kɔ́"
    ],
    "december": [
        "dsb",
        "sánzá ya zómi na míbalé"
    ],
    "monday": [
        "mokɔlɔ mwa yambo",
        "ybo"
    ],
    "tuesday": [
        "mbl",
        "mokɔlɔ mwa míbalé"
    ],
    "wednesday": [
        "mokɔlɔ mwa mísáto",
        "mst"
    ],
    "thursday": [
        "min",
        "mokɔlɔ ya mínéi"
    ],
    "friday": [
        "mokɔlɔ ya mítáno",
        "mtn"
    ],
    "saturday": [
        "mps",
        "mpɔ́sɔ"
    ],
    "sunday": [
        "eye",
        "eyenga"
    ],
    "am": [
        "ntɔ́ngɔ́"
    ],
    "pm": [
        "mpókwa"
    ],
    "year": [
        "mobú"
    ],
    "month": [
        "sánzá"
    ],
    "week": [
        "pɔ́sɔ"
    ],
    "day": [
        "mokɔlɔ"
    ],
    "hour": [
        "ngonga"
    ],
    "minute": [
        "monúti"
    ],
    "second": [
        "sɛkɔ́ndɛ"
    ],
    "relative-type": {
        "0 day ago": [
            "lɛlɔ́"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "lóbi elékí"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "lóbi ekoyâ"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "ln-AO": {
            "name": "ln-AO"
        },
        "ln-CF": {
            "name": "ln-CF"
        },
        "ln-CG": {
            "name": "ln-CG"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "nl",
    "date_order": "DMY",
    "january": [
        "jan",
        "januari"
    ],
    "february": [
        "feb",
        "februari"
    ],
    "march": [
        "maart",
        "mrt"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "juni"
    ],
    "july": [
        "jul",
        "juli"
    ],
    "august": [
        "aug",
        "augustus"
    ],
    "september": [
        "sep",
        "september"
    ],
    "october": [
        "okt",
        "oktober"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "dec",
        "december"
    ],
    "monday": [
        "ma",
        "maandag"
    ],
    "tuesday": [
        "di",
        "dinsdag"
    ],
    "wednesday": [
        "wo",
        "woensdag"
    ],
    "thursday": [
        "do",
        "donderdag"
    ],
    "friday": [
        "vr",
        "vrijdag"
    ],
    "saturday": [
        "za",
        "zaterdag"
    ],
    "sunday": [
        "zo",
        "zondag"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "jaar",
        "jr"
    ],
    "month": [
        "maand",
        "mnd",
        "maanden"
    ],
    "week": [
        "week",
        "wk",
        "weken"
    ],
    "day": [
        "dag",
        "dagen"
    ],
    "hour": [
        "uur"
    ],
    "minute": [
        "min",
        "minuut",
        "minuten"
    ],
    "second": [
        "s",
        "sec",
        "seconde",
        "seconden"
    ],
    "relative-type": {
        "0 day ago": [
            "vandaag"
        ],
        "0 hour ago": [
            "binnen een uur"
        ],
        "0 minute ago": [
            "binnen een minuut"
        ],
        "0 month ago": [
            "deze maand"
        ],
        "0 second ago": [
            "nu"
        ],
        "0 week ago": [
            "deze week"
        ],
        "0 year ago": [
            "dit jaar"
        ],
        "1 day ago": [
            "gisteren"
        ],
        "1 month ago": [
            "vorige maand"
        ],
        "1 week ago": [
            "vorige week"
        ],
        "1 year ago": [
            "vorig jaar",
            "vorige jaar"
        ],
        "in 1 day": [
            "morgen"
        ],
        "in 1 month": [
            "volgende maand"
        ],
        "in 1 week": [
            "volgende week"
        ],
        "in 1 year": [
            "volgend jaar"
        ],
        "2 day ago": [
            "eergisteren"
        ],
        "in 2 day": [
            "overmorgen"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) dag geleden",
            "(\\d+[.,]?\\d*) dagen geleden",
            "(\\d+[.,]?\\d*) dgn geleden"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) uur geleden"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min geleden",
            "(\\d+[.,]?\\d*) minuten geleden",
            "(\\d+[.,]?\\d*) minuut geleden"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) maand geleden",
            "(\\d+[.,]?\\d*) maanden geleden"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) sec geleden",
            "(\\d+[.,]?\\d*) seconde geleden",
            "(\\d+[.,]?\\d*) seconden geleden"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) week geleden",
            "(\\d+[.,]?\\d*) weken geleden"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) jaar geleden"
        ],
        "in \\1 day": [
            "over (\\d+[.,]?\\d*) dag",
            "over (\\d+[.,]?\\d*) dagen",
            "over (\\d+[.,]?\\d*) dgn"
        ],
        "in \\1 hour": [
            "over (\\d+[.,]?\\d*) uur"
        ],
        "in \\1 minute": [
            "over (\\d+[.,]?\\d*) min",
            "over (\\d+[.,]?\\d*) minuten",
            "over (\\d+[.,]?\\d*) minuut"
        ],
        "in \\1 month": [
            "over (\\d+[.,]?\\d*) maand",
            "over (\\d+[.,]?\\d*) maanden"
        ],
        "in \\1 second": [
            "over (\\d+[.,]?\\d*) sec",
            "over (\\d+[.,]?\\d*) seconde",
            "over (\\d+[.,]?\\d*) seconden"
        ],
        "in \\1 week": [
            "over (\\d+[.,]?\\d*) week",
            "over (\\d+[.,]?\\d*) weken"
        ],
        "in \\1 year": [
            "over (\\d+[.,]?\\d*) jaar"
        ]
    },
    "locale_specific": {
        "nl-AW": {
            "name": "nl-AW"
        },
        "nl-BE": {
            "name": "nl-BE"
        },
        "nl-BQ": {
            "name": "nl-BQ"
        },
        "nl-CW": {
            "name": "nl-CW"
        },
        "nl-SR": {
            "name": "nl-SR"
        },
        "nl-SX": {
            "name": "nl-SX"
        }
    },
    "skip": [
        "om",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "geleden"
    ],
    "in": [
        "in"
    ]
}



info = {
    "name": "et",
    "date_order": "DMY",
    "january": [
        "jaan",
        "jaanuar"
    ],
    "february": [
        "veebr",
        "veebruar"
    ],
    "march": [
        "märts"
    ],
    "april": [
        "apr",
        "aprill"
    ],
    "may": [
        "mai"
    ],
    "june": [
        "juuni"
    ],
    "july": [
        "juuli"
    ],
    "august": [
        "aug",
        "august"
    ],
    "september": [
        "sept",
        "september"
    ],
    "october": [
        "okt",
        "oktoober"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "dets",
        "detsember"
    ],
    "monday": [
        "e",
        "esmaspäev"
    ],
    "tuesday": [
        "t",
        "teisipäev"
    ],
    "wednesday": [
        "k",
        "kolmapäev"
    ],
    "thursday": [
        "n",
        "neljapäev"
    ],
    "friday": [
        "r",
        "reede"
    ],
    "saturday": [
        "l",
        "laupäev"
    ],
    "sunday": [
        "p",
        "pühapäev"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "a",
        "aasta"
    ],
    "month": [
        "k",
        "kuu"
    ],
    "week": [
        "näd",
        "nädal"
    ],
    "day": [
        "p",
        "päev"
    ],
    "hour": [
        "t",
        "tund"
    ],
    "minute": [
        "min",
        "minut"
    ],
    "second": [
        "s",
        "sek",
        "sekund"
    ],
    "relative-type": {
        "0 day ago": [
            "täna"
        ],
        "0 hour ago": [
            "praegusel tunnil"
        ],
        "0 minute ago": [
            "praegusel minutil"
        ],
        "0 month ago": [
            "käesolev kuu"
        ],
        "0 second ago": [
            "nüüd"
        ],
        "0 week ago": [
            "käesolev nädal"
        ],
        "0 year ago": [
            "käesolev aasta"
        ],
        "1 day ago": [
            "eile"
        ],
        "1 month ago": [
            "eelmine kuu"
        ],
        "1 week ago": [
            "eelmine nädal"
        ],
        "1 year ago": [
            "eelmine aasta"
        ],
        "in 1 day": [
            "homme"
        ],
        "in 1 month": [
            "järgmine kuu"
        ],
        "in 1 week": [
            "järgmine nädal"
        ],
        "in 1 year": [
            "järgmine aasta"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) p eest",
            "(\\d+[.,]?\\d*) päeva eest"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) t eest",
            "(\\d+[.,]?\\d*) tunni eest"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min eest",
            "(\\d+[.,]?\\d*) minuti eest"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) k eest",
            "(\\d+[.,]?\\d*) kuu eest"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) s eest",
            "(\\d+[.,]?\\d*) sek eest",
            "(\\d+[.,]?\\d*) sekundi eest"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) näd eest",
            "(\\d+[.,]?\\d*) nädala eest"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) a eest",
            "(\\d+[.,]?\\d*) aasta eest"
        ],
        "in \\1 day": [
            "(\\d+[.,]?\\d*) p pärast",
            "(\\d+[.,]?\\d*) päeva pärast"
        ],
        "in \\1 hour": [
            "(\\d+[.,]?\\d*) t pärast",
            "(\\d+[.,]?\\d*) tunni pärast"
        ],
        "in \\1 minute": [
            "(\\d+[.,]?\\d*) min pärast",
            "(\\d+[.,]?\\d*) minuti pärast"
        ],
        "in \\1 month": [
            "(\\d+[.,]?\\d*) k pärast",
            "(\\d+[.,]?\\d*) kuu pärast"
        ],
        "in \\1 second": [
            "(\\d+[.,]?\\d*) s pärast",
            "(\\d+[.,]?\\d*) sek pärast",
            "(\\d+[.,]?\\d*) sekundi pärast"
        ],
        "in \\1 week": [
            "(\\d+[.,]?\\d*) näd pärast",
            "(\\d+[.,]?\\d*) nädala pärast"
        ],
        "in \\1 year": [
            "(\\d+[.,]?\\d*) a pärast",
            "(\\d+[.,]?\\d*) aasta pärast"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



from datetime import datetime, time, timezone

import regex as re
from dateutil.relativedelta import relativedelta
from tzlocal import get_localzone

from dateparser.utils import apply_timezone, localize_timezone, strip_braces

from .parser import time_parser
from .timezone_parser import pop_tz_offset_from_string

_UNITS = r"decade|year|month|week|day|hour|minute|second"
PATTERN = re.compile(r"([+-]?\d+[.,]?\d*)\s*(%s)\b" % _UNITS, re.I | re.S | re.U)


class FreshnessDateDataParser:
    """Parses date string like "1 year, 2 months ago" and "3 hours, 50 minutes ago" """

    def _are_all_words_units(self, date_string):
        skip = [_UNITS, r"ago|in|\d+", r":|[ap]m"]

        date_string = re.sub(r"\s+", " ", date_string.strip())

        words = [x for x in re.split(r"\W", date_string) if x]
        words = [x for x in words if not re.match(r"%s" % "|".join(skip), x)]
        return not words

    def _parse_time(self, date_string, settings):
        """Attempts to parse time part of date strings like '1 day ago, 2 PM'"""
        date_string = PATTERN.sub("", date_string)
        date_string = re.sub(r"\b(?:ago|in)\b", "", date_string)
        try:
            return time_parser(date_string)
        except Exception:
            pass

    def get_local_tz(self):
        return get_localzone()

    def parse(self, date_string, settings):
        date_string = strip_braces(date_string)
        date_string, ptz = pop_tz_offset_from_string(date_string)
        _time = self._parse_time(date_string, settings)

        _settings_tz = settings.TIMEZONE.lower()

        def apply_time(dateobj, timeobj):
            if not isinstance(_time, time):
                return dateobj

            return dateobj.replace(
                hour=timeobj.hour,
                minute=timeobj.minute,
                second=timeobj.second,
                microsecond=timeobj.microsecond,
            )

        if settings.RELATIVE_BASE:
            now = settings.RELATIVE_BASE

            if "local" not in _settings_tz:
                now = localize_timezone(now, settings.TIMEZONE)

            if ptz:
                if now.tzinfo:
                    now = now.astimezone(ptz)
                else:
                    if hasattr(ptz, "localize"):
                        now = ptz.localize(now)
                    else:
                        now = now.replace(tzinfo=ptz)

            if not now.tzinfo:
                now = now.replace(tzinfo=self.get_local_tz())

        elif ptz:
            localized_now = datetime.now(ptz)

            if "local" in _settings_tz:
                now = localized_now
            else:
                now = apply_timezone(localized_now, settings.TIMEZONE)

        else:
            if "local" not in _settings_tz:
                utc_dt = datetime.now(tz=timezone.utc)
                now = apply_timezone(utc_dt, settings.TIMEZONE)
            else:
                now = datetime.now(self.get_local_tz())

        date, period = self._parse_date(date_string, now, settings.PREFER_DATES_FROM)

        if date:
            old_date = date
            date = apply_time(date, _time)
            if settings.RETURN_TIME_AS_PERIOD and old_date != date:
                period = "time"

            if settings.TO_TIMEZONE:
                date = apply_timezone(date, settings.TO_TIMEZONE)

            if not settings.RETURN_AS_TIMEZONE_AWARE or (
                settings.RETURN_AS_TIMEZONE_AWARE
                and "default" == settings.RETURN_AS_TIMEZONE_AWARE
                and not ptz
            ):
                date = date.replace(tzinfo=None)

        return date, period

    def _parse_date(self, date_string, now, prefer_dates_from):
        if not self._are_all_words_units(date_string):
            return None, None

        result = self.get_kwargs(date_string)
        if isinstance(result, tuple):
            kwargs, explicit_signs = result
        else:
            kwargs = result
            explicit_signs = {}

        if not kwargs:
            return None, None
        period = "day"
        if "days" not in kwargs:
            for k in ["weeks", "months", "years"]:
                if k in kwargs:
                    period = k[:-1]
                    break

        going_forward = (
            re.search(r"\bin\b", date_string)
            or re.search(r"\bfuture\b", prefer_dates_from)
            and not re.search(r"\bago\b", date_string)
        )

        adjusted_kwargs = {}
        for key, value in kwargs.items():
            if explicit_signs.get(key, False):
                adjusted_kwargs[key] = value
            else:
                if going_forward:
                    adjusted_kwargs[key] = value
                else:
                    adjusted_kwargs[key] = -value

        td = relativedelta(**adjusted_kwargs)

        date = now + td

        return date, period

    def get_kwargs(self, date_string):
        m = PATTERN.findall(date_string)
        if not m:
            return {}

        kwargs = {}
        explicit_signs = {}

        for num, unit in m:
            has_explicit_sign = num.startswith("+") or num.startswith("-")
            explicit_signs[unit + "s"] = has_explicit_sign
            kwargs[unit + "s"] = float(num.replace(",", "."))

        if "decades" in kwargs:
            kwargs["years"] = 10 * kwargs["decades"] + kwargs.get("years", 0)
            if "decades" in explicit_signs:
                explicit_signs["years"] = explicit_signs["decades"]
            del kwargs["decades"]
            explicit_signs.pop("decades", None)

        return kwargs, explicit_signs

    def get_date_data(self, date_string, settings=None):
        from dateparser.date import DateData

        date, period = self.parse(date_string, settings)
        return DateData(date_obj=date, period=period)


freshness_date_parser = FreshnessDateDataParser()



info = {
    "name": "ak",
    "date_order": "YMD",
    "january": [
        "s-ɔ",
        "sanda-ɔpɛpɔn"
    ],
    "february": [
        "k-ɔ",
        "kwakwar-ɔgyefuo"
    ],
    "march": [
        "e-ɔ",
        "ebɔw-ɔbenem"
    ],
    "april": [
        "e-o",
        "ebɔbira-oforisuo"
    ],
    "may": [
        "e-k",
        "esusow aketseaba-kɔtɔnimba"
    ],
    "june": [
        "o-a",
        "obirade-ayɛwohomumu"
    ],
    "july": [
        "a-k",
        "ayɛwoho-kitawonsa"
    ],
    "august": [
        "d-ɔ",
        "difuu-ɔsandaa"
    ],
    "september": [
        "f-ɛ",
        "fankwa-ɛbɔ"
    ],
    "october": [
        "ɔ-a",
        "ɔbɛsɛ-ahinime"
    ],
    "november": [
        "ɔ-o",
        "ɔberɛfɛw-obubuo"
    ],
    "december": [
        "m-ɔ",
        "mumu-ɔpɛnimba"
    ],
    "monday": [
        "dwo",
        "dwowda"
    ],
    "tuesday": [
        "ben",
        "benada"
    ],
    "wednesday": [
        "wuk",
        "wukuda"
    ],
    "thursday": [
        "yaw",
        "yawda"
    ],
    "friday": [
        "fia",
        "fida"
    ],
    "saturday": [
        "mem",
        "memeneda"
    ],
    "sunday": [
        "kwe",
        "kwesida"
    ],
    "am": [
        "an"
    ],
    "pm": [
        "ew"
    ],
    "year": [
        "afe"
    ],
    "month": [
        "bosome"
    ],
    "week": [
        "dapɛn"
    ],
    "day": [
        "da"
    ],
    "hour": [
        "dɔnhwer"
    ],
    "minute": [
        "sema"
    ],
    "second": [
        "sɛkɛnd"
    ],
    "relative-type": {
        "0 day ago": [
            "ndɛ"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "ndeda"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "ɔkyena"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "om",
    "date_order": "DMY",
    "january": [
        "ama",
        "amajjii"
    ],
    "february": [
        "gur",
        "guraandhala"
    ],
    "march": [
        "bit",
        "bitooteessa"
    ],
    "april": [
        "elb",
        "elba"
    ],
    "may": [
        "caamsa",
        "cam"
    ],
    "june": [
        "wax",
        "waxabajjii"
    ],
    "july": [
        "ado",
        "adooleessa"
    ],
    "august": [
        "hag",
        "hagayya"
    ],
    "september": [
        "ful",
        "fuulbana"
    ],
    "october": [
        "onk",
        "onkololeessa"
    ],
    "november": [
        "sad",
        "sadaasa"
    ],
    "december": [
        "mud",
        "muddee"
    ],
    "monday": [
        "wiixata",
        "wix"
    ],
    "tuesday": [
        "qib",
        "qibxata"
    ],
    "wednesday": [
        "rob",
        "roobii"
    ],
    "thursday": [
        "kam",
        "kamiisa"
    ],
    "friday": [
        "jim",
        "jimaata"
    ],
    "saturday": [
        "san",
        "sanbata"
    ],
    "sunday": [
        "dil",
        "dilbata"
    ],
    "am": [
        "wd"
    ],
    "pm": [
        "wb"
    ],
    "year": [
        "year"
    ],
    "month": [
        "month"
    ],
    "week": [
        "week"
    ],
    "day": [
        "day"
    ],
    "hour": [
        "hour"
    ],
    "minute": [
        "minute"
    ],
    "second": [
        "second"
    ],
    "relative-type": {
        "0 day ago": [
            "today"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "yesterday"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "tomorrow"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {
        "om-KE": {
            "name": "om-KE"
        }
    },
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



import os

import fasttext

from dateparser_cli.exceptions import FastTextModelNotFoundException
from dateparser_cli.fasttext_manager import fasttext_downloader
from dateparser_cli.utils import create_data_model_home, dateparser_model_home

_supported_models = ["large.bin", "small.bin"]
_DEFAULT_MODEL = "small"


class _FastTextCache:
    model = None


def _load_fasttext_model():
    if _FastTextCache.model:
        return _FastTextCache.model
    create_data_model_home()
    downloaded_models = [
        file for file in os.listdir(dateparser_model_home) if file in _supported_models
    ]
    if not downloaded_models:
        fasttext_downloader(_DEFAULT_MODEL)
        return _load_fasttext_model()
    model_path = os.path.join(dateparser_model_home, downloaded_models[0])
    if not os.path.isfile(model_path):
        raise FastTextModelNotFoundException("Fasttext model file not found")
    _FastTextCache.model = fasttext.load_model(model_path)
    return _FastTextCache.model


def detect_languages(text, confidence_threshold):
    _language_parser = _load_fasttext_model()
    text = text.replace("\n", " ").replace("\r", "")
    language_codes = []
    parser_data = _language_parser.predict(text)
    for idx, language_probability in enumerate(parser_data[1]):
        if language_probability > confidence_threshold:
            language_code = parser_data[0][idx].replace("__label__", "")
            language_codes.append(language_code)
    return language_codes



info = {
    "name": "nn",
    "date_order": "DMY",
    "january": [
        "jan",
        "januar"
    ],
    "february": [
        "feb",
        "februar"
    ],
    "march": [
        "mar",
        "mars"
    ],
    "april": [
        "apr",
        "april"
    ],
    "may": [
        "mai"
    ],
    "june": [
        "jun",
        "juni"
    ],
    "july": [
        "jul",
        "juli"
    ],
    "august": [
        "aug",
        "august"
    ],
    "september": [
        "sep",
        "september"
    ],
    "october": [
        "okt",
        "oktober"
    ],
    "november": [
        "nov",
        "november"
    ],
    "december": [
        "des",
        "desember"
    ],
    "monday": [
        "må",
        "mån",
        "måndag"
    ],
    "tuesday": [
        "ty",
        "tys",
        "tysdag"
    ],
    "wednesday": [
        "on",
        "ons",
        "onsdag"
    ],
    "thursday": [
        "to",
        "tor",
        "torsdag"
    ],
    "friday": [
        "fr",
        "fre",
        "fredag"
    ],
    "saturday": [
        "la",
        "lau",
        "laurdag"
    ],
    "sunday": [
        "sø",
        "søn",
        "søndag"
    ],
    "am": [
        "fm",
        "formiddag"
    ],
    "pm": [
        "em",
        "ettermiddag"
    ],
    "year": [
        "år"
    ],
    "month": [
        "månad"
    ],
    "week": [
        "veke"
    ],
    "day": [
        "dag"
    ],
    "hour": [
        "time"
    ],
    "minute": [
        "minutt"
    ],
    "second": [
        "sekund"
    ],
    "relative-type": {
        "0 day ago": [
            "i dag"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "i går"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "i morgon"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "for (\\d+[.,]?\\d*) døgn siden"
        ],
        "\\1 hour ago": [
            "for (\\d+[.,]?\\d*) time siden",
            "for (\\d+[.,]?\\d*) timer siden"
        ],
        "\\1 minute ago": [
            "for (\\d+[.,]?\\d*) minutt siden",
            "for (\\d+[.,]?\\d*) minutter siden"
        ],
        "\\1 month ago": [
            "for (\\d+[.,]?\\d*) måned siden",
            "for (\\d+[.,]?\\d*) måneder siden"
        ],
        "\\1 second ago": [
            "for (\\d+[.,]?\\d*) sekund siden",
            "for (\\d+[.,]?\\d*) sekunder siden"
        ],
        "\\1 week ago": [
            "for (\\d+[.,]?\\d*) uke siden",
            "for (\\d+[.,]?\\d*) uker siden"
        ],
        "\\1 year ago": [
            "for (\\d+[.,]?\\d*) år siden"
        ],
        "in \\1 day": [
            "om (\\d+[.,]?\\d*) døgn"
        ],
        "in \\1 hour": [
            "om (\\d+[.,]?\\d*) time",
            "om (\\d+[.,]?\\d*) timer"
        ],
        "in \\1 minute": [
            "om (\\d+[.,]?\\d*) minutt",
            "om (\\d+[.,]?\\d*) minutter"
        ],
        "in \\1 month": [
            "om (\\d+[.,]?\\d*) måned",
            "om (\\d+[.,]?\\d*) måneder"
        ],
        "in \\1 second": [
            "om (\\d+[.,]?\\d*) sekund",
            "om (\\d+[.,]?\\d*) sekunder"
        ],
        "in \\1 week": [
            "om (\\d+[.,]?\\d*) uke",
            "om (\\d+[.,]?\\d*) uker"
        ],
        "in \\1 year": [
            "om (\\d+[.,]?\\d*) år"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "be",
    "date_order": "DMY",
    "january": [
        "сту",
        "студзень",
        "студзеня",
        "Стд"
    ],
    "february": [
        "лют",
        "лютага",
        "люты"
    ],
    "march": [
        "сак",
        "сакавік",
        "сакавіка"
    ],
    "april": [
        "кра",
        "красавік",
        "красавіка",
        "Крс"
    ],
    "may": [
        "май",
        "мая",
        "Траўня",
        "Тра"
    ],
    "june": [
        "чэр",
        "чэрвень",
        "чэрвеня"
    ],
    "july": [
        "ліп",
        "ліпень",
        "ліпеня"
    ],
    "august": [
        "жні",
        "жнівень",
        "жніўня",
        "Жнівеня",
        "Жнв"
    ],
    "september": [
        "вер",
        "верасень",
        "верасня",
        "Врс"
    ],
    "october": [
        "кас",
        "кастрычнік",
        "кастрычніка",
        "Кст"
    ],
    "november": [
        "ліс",
        "лістапад",
        "лістапада",
        "Ліс"
    ],
    "december": [
        "сне",
        "снежань",
        "снежня",
        "Снж"
    ],
    "monday": [
        "панядзелак",
        "пн",
        "Пнд"
    ],
    "tuesday": [
        "аў",
        "аўторак",
        "Аўт"
    ],
    "wednesday": [
        "серада",
        "ср"
    ],
    "thursday": [
        "чацвер",
        "чц",
        "Чцв",
        "Чв"
    ],
    "friday": [
        "пт",
        "пятніца",
        "Пят"
    ],
    "saturday": [
        "сб",
        "субота",
        "Суб"
    ],
    "sunday": [
        "нд",
        "нядзеля",
        "Няд"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "г",
        "год",
        "гады",
        "года",
        "гадоў"
    ],
    "month": [
        "мес",
        "месяц",
        "месяца",
        "месяцы",
        "месяцаў"
    ],
    "week": [
        "тыд",
        "тыдзень",
        "тыдня",
        "тыдні",
        "тыдняў"
    ],
    "day": [
        "д",
        "дзень",
        "дні",
        "дзён",
        "дзен"
    ],
    "hour": [
        "гадз",
        "гадзіна",
        "гадзіны",
        "гадзіну",
        "гадзін"
    ],
    "minute": [
        "хв",
        "хвіліна",
        "хвілін",
        "хвіліны",
        "хвіліну",
        "хвіл"
    ],
    "second": [
        "с",
        "секунда",
        "секунды",
        "секунду",
        "секунд",
        "сек"
    ],
    "relative-type": {
        "0 day ago": [
            "сёння",
            "сення"
        ],
        "0 hour ago": [
            "у гэту гадзіну"
        ],
        "0 minute ago": [
            "у гэту хвіліну"
        ],
        "0 month ago": [
            "у гэтым месяцы"
        ],
        "0 second ago": [
            "цяпер"
        ],
        "0 week ago": [
            "на гэтым тыдні"
        ],
        "0 year ago": [
            "у гэтым годзе"
        ],
        "1 day ago": [
            "учора",
            "ўчора"
        ],
        "1 month ago": [
            "у мінулым месяцы"
        ],
        "1 week ago": [
            "на мінулым тыдні"
        ],
        "1 year ago": [
            "у мінулым годзе"
        ],
        "in 1 day": [
            "заўтра"
        ],
        "in 1 month": [
            "у наступным месяцы"
        ],
        "in 1 week": [
            "на наступным тыдні"
        ],
        "in 1 year": [
            "у наступным годзе"
        ],
        "2 day ago": [
            "пазаўчора"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) д таму",
            "(\\d+[.,]?\\d*) дзень таму",
            "(\\d+[.,]?\\d*) дня таму"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) гадз таму",
            "(\\d+[.,]?\\d*) гадзіну таму",
            "(\\d+[.,]?\\d*) гадзіны таму"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) хв таму",
            "(\\d+[.,]?\\d*) хвіліну таму",
            "(\\d+[.,]?\\d*) хвіліны таму"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) мес таму",
            "(\\d+[.,]?\\d*) месяц таму",
            "(\\d+[.,]?\\d*) месяца таму"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) с таму",
            "(\\d+[.,]?\\d*) секунду таму",
            "(\\d+[.,]?\\d*) секунды таму"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) тыд таму",
            "(\\d+[.,]?\\d*) тыдзень таму",
            "(\\d+[.,]?\\d*) тыдня таму"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) г таму",
            "(\\d+[.,]?\\d*) год таму",
            "(\\d+[.,]?\\d*) года таму"
        ],
        "in \\1 day": [
            "праз (\\d+[.,]?\\d*) д",
            "праз (\\d+[.,]?\\d*) дзень",
            "праз (\\d+[.,]?\\d*) дня"
        ],
        "in \\1 hour": [
            "праз (\\d+[.,]?\\d*) гадз",
            "праз (\\d+[.,]?\\d*) гадзіну",
            "праз (\\d+[.,]?\\d*) гадзіны"
        ],
        "in \\1 minute": [
            "праз (\\d+[.,]?\\d*) хв",
            "праз (\\d+[.,]?\\d*) хвіліну",
            "праз (\\d+[.,]?\\d*) хвіліны"
        ],
        "in \\1 month": [
            "праз (\\d+[.,]?\\d*) мес",
            "праз (\\d+[.,]?\\d*) месяц",
            "праз (\\d+[.,]?\\d*) месяца"
        ],
        "in \\1 second": [
            "праз (\\d+[.,]?\\d*) с",
            "праз (\\d+[.,]?\\d*) секунду",
            "праз (\\d+[.,]?\\d*) секунды"
        ],
        "in \\1 week": [
            "праз (\\d+[.,]?\\d*) тыд",
            "праз (\\d+[.,]?\\d*) тыдзень",
            "праз (\\d+[.,]?\\d*) тыдня"
        ],
        "in \\1 year": [
            "праз (\\d+[.,]?\\d*) г",
            "праз (\\d+[.,]?\\d*) год",
            "праз (\\d+[.,]?\\d*) года"
        ]
    },
    "locale_specific": {},
    "skip": [
        "каля",
        "у",
        "і",
        "ў",
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ],
    "sentence_splitter_group": 1,
    "ago": [
        "таму назад",
        "таму",
        "назад"
    ],
    "in": [
        "на працягу"
    ],
    "simplifications": [
        {
            "^гадзіна": "1 гадзіна"
        },
        {
            "гадзіну": "1 гадзіну"
        },
        {
            "^хвіліну": "1 хвіліну"
        },
        {
            "^секунду": "1 секунду"
        },
        {
            "некалькі секунд": "44 секунды"
        },
        {
            "некалькі хвілін": "2 хвіліны"
        },
        {
            "(\\d+[.,]?\\d*)\\s*гадзін\\s(\\d+[.,]?\\d*)\\s*хвілін": "\\1:\\2"
        }
    ]
}



info = {
    "name": "luy",
    "date_order": "DMY",
    "january": [
        "jan",
        "januari"
    ],
    "february": [
        "feb",
        "februari"
    ],
    "march": [
        "machi",
        "mar"
    ],
    "april": [
        "apr",
        "aprili"
    ],
    "may": [
        "mei"
    ],
    "june": [
        "jun",
        "juni"
    ],
    "july": [
        "jul",
        "julai"
    ],
    "august": [
        "ago",
        "agosti"
    ],
    "september": [
        "sep",
        "septemba"
    ],
    "october": [
        "okt",
        "oktoba"
    ],
    "november": [
        "nov",
        "novemba"
    ],
    "december": [
        "des",
        "desemba"
    ],
    "monday": [
        "j3",
        "jumatatu"
    ],
    "tuesday": [
        "j4",
        "jumanne"
    ],
    "wednesday": [
        "j5",
        "jumatano"
    ],
    "thursday": [
        "al",
        "murwa wa kanne"
    ],
    "friday": [
        "ij",
        "murwa wa katano"
    ],
    "saturday": [
        "j1",
        "jumamosi"
    ],
    "sunday": [
        "j2",
        "jumapiri"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "muhiga"
    ],
    "month": [
        "mweri"
    ],
    "week": [
        "risiza"
    ],
    "day": [
        "ridiku"
    ],
    "hour": [
        "isaa"
    ],
    "minute": [
        "idagika"
    ],
    "second": [
        "sekunde"
    ],
    "relative-type": {
        "0 day ago": [
            "lero"
        ],
        "0 hour ago": [
            "this hour"
        ],
        "0 minute ago": [
            "this minute"
        ],
        "0 month ago": [
            "this month"
        ],
        "0 second ago": [
            "now"
        ],
        "0 week ago": [
            "this week"
        ],
        "0 year ago": [
            "this year"
        ],
        "1 day ago": [
            "mgorova"
        ],
        "1 month ago": [
            "last month"
        ],
        "1 week ago": [
            "last week"
        ],
        "1 year ago": [
            "last year"
        ],
        "in 1 day": [
            "mgamba"
        ],
        "in 1 month": [
            "next month"
        ],
        "in 1 week": [
            "next week"
        ],
        "in 1 year": [
            "next year"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}



info = {
    "name": "fil",
    "date_order": "MDY",
    "january": [
        "ene",
        "enero"
    ],
    "february": [
        "peb",
        "pebrero"
    ],
    "march": [
        "mar",
        "marso"
    ],
    "april": [
        "abr",
        "abril"
    ],
    "may": [
        "may",
        "mayo"
    ],
    "june": [
        "hun",
        "hunyo"
    ],
    "july": [
        "hul",
        "hulyo"
    ],
    "august": [
        "ago",
        "agosto"
    ],
    "september": [
        "set",
        "setyembre"
    ],
    "october": [
        "okt",
        "oktubre"
    ],
    "november": [
        "nob",
        "nobyembre"
    ],
    "december": [
        "dis",
        "disyembre"
    ],
    "monday": [
        "lun",
        "lunes"
    ],
    "tuesday": [
        "mar",
        "martes"
    ],
    "wednesday": [
        "miy",
        "miyerkules"
    ],
    "thursday": [
        "huw",
        "huwebes"
    ],
    "friday": [
        "biy",
        "biyernes"
    ],
    "saturday": [
        "sab",
        "sabado"
    ],
    "sunday": [
        "lin",
        "linggo"
    ],
    "am": [
        "am"
    ],
    "pm": [
        "pm"
    ],
    "year": [
        "taon"
    ],
    "month": [
        "buwan"
    ],
    "week": [
        "linggo"
    ],
    "day": [
        "araw"
    ],
    "hour": [
        "oras"
    ],
    "minute": [
        "min",
        "minuto"
    ],
    "second": [
        "seg",
        "segundo"
    ],
    "relative-type": {
        "0 day ago": [
            "ngayong araw"
        ],
        "0 hour ago": [
            "ngayong oras"
        ],
        "0 minute ago": [
            "sa minutong ito"
        ],
        "0 month ago": [
            "ngayong buwan"
        ],
        "0 second ago": [
            "ngayon"
        ],
        "0 week ago": [
            "ngayong linggo",
            "sa linggong ito"
        ],
        "0 year ago": [
            "ngayong taon"
        ],
        "1 day ago": [
            "kahapon"
        ],
        "1 month ago": [
            "nakaraang buwan"
        ],
        "1 week ago": [
            "nakalipas na linggo",
            "nakaraang linggo"
        ],
        "1 year ago": [
            "nakaraang taon"
        ],
        "in 1 day": [
            "bukas"
        ],
        "in 1 month": [
            "susunod na buwan"
        ],
        "in 1 week": [
            "susunod na linggo"
        ],
        "in 1 year": [
            "susunod na taon"
        ]
    },
    "relative-type-regex": {
        "\\1 day ago": [
            "(\\d+[.,]?\\d*) araw ang nakalipas"
        ],
        "\\1 hour ago": [
            "(\\d+[.,]?\\d*) oras ang nakalipas",
            "(\\d+[.,]?\\d*) oras nakalipas"
        ],
        "\\1 minute ago": [
            "(\\d+[.,]?\\d*) min ang nakalipas",
            "(\\d+[.,]?\\d*) minuto ang nakalipas"
        ],
        "\\1 month ago": [
            "(\\d+[.,]?\\d*) buwan ang nakalipas"
        ],
        "\\1 second ago": [
            "(\\d+[.,]?\\d*) seg ang nakalipas",
            "(\\d+[.,]?\\d*) seg nakalipas",
            "(\\d+[.,]?\\d*) segundo ang nakalipas"
        ],
        "\\1 week ago": [
            "(\\d+[.,]?\\d*) linggo ang nakalipas"
        ],
        "\\1 year ago": [
            "(\\d+[.,]?\\d*) taon ang nakalipas"
        ],
        "in \\1 day": [
            "sa (\\d+[.,]?\\d*) araw"
        ],
        "in \\1 hour": [
            "sa (\\d+[.,]?\\d*) oras"
        ],
        "in \\1 minute": [
            "sa (\\d+[.,]?\\d*) min",
            "sa (\\d+[.,]?\\d*) minuto"
        ],
        "in \\1 month": [
            "sa (\\d+[.,]?\\d*) buwan"
        ],
        "in \\1 second": [
            "sa (\\d+[.,]?\\d*) seg",
            "sa (\\d+[.,]?\\d*) segundo"
        ],
        "in \\1 week": [
            "sa (\\d+[.,]?\\d*) linggo"
        ],
        "in \\1 year": [
            "sa (\\d+[.,]?\\d*) taon"
        ]
    },
    "locale_specific": {},
    "skip": [
        " ",
        "'",
        ",",
        "-",
        ".",
        "/",
        ";",
        "@",
        "[",
        "]",
        "|",
        "，"
    ]
}
